@@ -63,6 +63,16 @@
     #include <solid/powermanagement.h>
 #endif
 
+
+KGet::TransferData::TransferData(const KUrl &source, const KUrl &destination, const QString& group, bool doStart, const QDomElement *element)
+  : src(source),
+    dest(destination),
+    groupName(group),
+    start(doStart),
+    e(element)
+{
+}
+
 /**
  * This is our KGet class. This is where the user's transfers and searches are
  * stored and organized.
@@ -100,28 +110,21 @@ bool KGet::addGroup(const QString& groupName)
     return true;
 }
 
-void KGet::delGroup(TransferGroupHandler * group, bool askUser)
+void KGet::delGroup(TransferGroupHandler *group, bool askUser)
 {
-    TransferGroup * g = group->m_group;
-    if (group)
-    {
-        if (askUser) {
-            QWidget * configDialog = KConfigDialog::exists("preferences");
-            if (KMessageBox::warningYesNo(configDialog ? configDialog : m_mainWindow,
-                           i18n("Are you sure that you want to remove the group named %1?", g->name()),
-                           i18n("Remove Group"),
-                           KStandardGuiItem::remove(), KStandardGuiItem::cancel()) != KMessageBox::Yes)
-                return;
-        }
-        JobQueue::iterator begin = g->begin();
-        JobQueue::iterator end = g->end();
-        for (; begin != end; begin++) {
-            Transfer *transfer = static_cast<Transfer*>(*begin);
-            m_transferTreeModel->delTransfer(transfer);//Unregister transfers from model first, they will get deleted by the group when destructing
-        }
-        m_transferTreeModel->delGroup(g);
-        g->deleteLater();
+    TransferGroup *g = group->m_group;
+
+    if (askUser) {
+        QWidget *configDialog = KConfigDialog::exists("preferences");
+        if (KMessageBox::warningYesNo(configDialog ? configDialog : m_mainWindow,
+                        i18n("Are you sure that you want to remove the group named %1?", g->name()),
+                        i18n("Remove Group"),
+                        KStandardGuiItem::remove(), KStandardGuiItem::cancel()) != KMessageBox::Yes)
+            return;
     }
+
+    m_transferTreeModel->delGroup(g);
+    g->deleteLater();
 }
 
 void KGet::delGroups(QList<TransferGroupHandler*> groups, bool askUser)
@@ -245,22 +248,32 @@ TransferHandler * KGet::addTransfer(KUrl srcUrl, QString destDir, QString sugges
 
     if (destUrl == KUrl())
         return 0;
-    
-    return createTransfer(srcUrl, destUrl, groupName, start);
+
+    TransferHandler *transfer = createTransfer(srcUrl, destUrl, groupName, start);
+    if (transfer) {
+        KGet::showNotification(m_mainWindow, "added",
+                                i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
+                                "kget", i18n("Download added"));
+    }
+
+    return transfer;
 }
 
-TransferHandler * KGet::addTransfer(const QDomElement& e, const QString& groupName)
+QList<TransferHandler*> KGet::addTransfers(const QList<QDomElement> &elements, const QString &groupName)
 {
-    //We need to read these attributes now in order to know which transfer
-    //plugin to use.
-    KUrl srcUrl = KUrl( e.attribute("Source") );
-    KUrl destUrl = KUrl( e.attribute("Dest") );
+    QList<TransferData> data;
 
-    kDebug(5001) << " src= " << srcUrl.url()
-              << " dest= " << destUrl.url() 
-              << " group= "<< groupName << endl;
+    foreach(const QDomElement &e, elements) {
+        //We need to read these attributes now in order to know which transfer
+        //plugin to use.
+        KUrl srcUrl = KUrl(e.attribute("Source"));
+        KUrl destUrl = KUrl(e.attribute("Dest"));
+        data << TransferData(srcUrl, destUrl, groupName, false, &e);
 
-    return createTransfer(srcUrl, destUrl, groupName, false, &e);
+        kDebug(5001) << "src=" << srcUrl << " dest=" << destUrl << " group=" << groupName;
+    }
+
+    return createTransfers(data);
 }
 
 const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString destDir, QString groupName, bool start)
@@ -302,6 +315,7 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
     it = urlsToDownload.begin();
     itEnd = urlsToDownload.end();
 
+    QList<TransferData> data;
     for ( ; it != itEnd; ++it )
     {
         if (destDir.isEmpty())
@@ -317,32 +331,49 @@ const QList<TransferHandler *> KGet::addTransfer(KUrl::List srcUrls, QString des
         if (destUrl == KUrl())
             continue;
 
-        TransferHandler * newTransfer = createTransfer(*it, destUrl, groupName, start);
-        
-        if(newTransfer)
-            addedTransfers.append(newTransfer);
+        data << TransferData(*it, destUrl, groupName, start);
     }
-    
-    return addedTransfers;
+
+    QList<TransferHandler*> transfers = createTransfers(data);
+    foreach (TransferHandler *transfer, transfers) {
+        KGet::showNotification(m_mainWindow, "added",
+                                i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
+                                "kget", i18n("Download added"));
+    }
+    return transfers;
 }
 
 
 bool KGet::delTransfer(TransferHandler * transfer)
 {
-    Transfer * t = transfer->m_transfer;
-    t->stop();
+    return delTransfers(QList<TransferHandler*>() << transfer);
+}
+
+bool KGet::delTransfers(QList<TransferHandler*> handlers)
+{
     if (!m_store) {
         m_store = TransferHistoryStore::getStore();
     }
-    m_store->saveItem(TransferHistoryItem(*t));
+    QList<Transfer*> transfers;
+    QList<TransferHistoryItem> historyItems;
+    foreach (TransferHandler *handler, handlers) {
+        Transfer *transfer = handler->m_transfer;
+        transfers << transfer;
+        historyItems << TransferHistoryItem(*transfer);
+        if (transfer->status() != Job::Finished) {//FIXME should this be here?
+            handler->stop();//TODO is this needed?
+            transfer->stop();//TODO is this needed?
+        }
 
-    // TransferHandler deinitializations
-    transfer->destroy();
-    // Transfer deinitializations (the deinit function is called by the destroy() function)
-    t->destroy();
-    
-    m_transferTreeModel->delTransfer(t);
-    delete t;
+        // TransferHandler deinitializations
+        handler->destroy();
+        // Transfer deinitializations (the deinit function is called by the destroy() function)
+        transfer->destroy();
+    }
+    m_store->saveItems(historyItems);
+
+    m_transferTreeModel->delTransfers(transfers);
+    qDeleteAll(transfers);
     return true;
 }
 
@@ -370,6 +401,8 @@ QList<TransferHandler *> KGet::selectedTransfers()
     QList<TransferHandler *> selectedTransfers;
 
     QModelIndexList selectedIndexes = m_selectionModel->selectedRows();
+    //sort the indexes as this can speed up operations like deleting etc.
+    qSort(selectedIndexes.begin(), selectedIndexes.end());
 
     foreach(const QModelIndex &currentIndex, selectedIndexes)
     {
@@ -588,6 +621,11 @@ bool KGet::schedulerRunning()
     return (m_scheduler->countRunningJobs() > 0);
 }
 
+void KGet::setSuspendScheduler(bool isSuspended)
+{
+    m_scheduler->setIsSuspended(isSuspended);
+}
+
 QList<TransferHandler*> KGet::allTransfers()
 {
     QList<TransferHandler*> transfers;
@@ -678,7 +716,7 @@ QList<TransferGroupHandler*> KGet::groupsFromExceptions(const KUrl &filename)
             }
 
             //now try with wildcards
-            if (!regExp.pattern().isEmpty() && !regExp.pattern().startsWith('*')) {
+            if (!regExp.pattern().isEmpty() && !regExp.pattern().contains('*')) {
                 regExp.setPattern('*' + regExp.pattern());
             }
 
@@ -742,10 +780,10 @@ KGet::KGet()
     m_transferTreeModel = new TransferTreeModel(m_scheduler);
     m_selectionModel = new TransferTreeSelectionModel(m_transferTreeModel);
 
-    QObject::connect(m_transferTreeModel, SIGNAL(transferAddedEvent(TransferHandler *, TransferGroupHandler *)),
-                     m_jobManager,        SLOT(slotTransferAdded(TransferHandler *, TransferGroupHandler *)));
-    QObject::connect(m_transferTreeModel, SIGNAL(transferAboutToBeRemovedEvent(TransferHandler *, TransferGroupHandler *)),
-                     m_jobManager,        SLOT(slotTransferAboutToBeRemoved(TransferHandler *, TransferGroupHandler *)));
+    QObject::connect(m_transferTreeModel, SIGNAL(transfersAddedEvent(QList<TransferHandler*>)),
+                     m_jobManager,        SLOT(slotTransfersAdded(QList<TransferHandler*>)));
+    QObject::connect(m_transferTreeModel, SIGNAL(transfersAboutToBeRemovedEvent(QList<TransferHandler*>)),
+                     m_jobManager,        SLOT(slotTransfersAboutToBeRemoved(QList<TransferHandler*>)));
     QObject::connect(m_transferTreeModel, SIGNAL(transfersChangedEvent(QMap<TransferHandler *, Transfer::ChangesFlags>)),
                      m_jobManager,        SLOT(slotTransfersChanged(QMap<TransferHandler *, Transfer::ChangesFlags>)));
             
@@ -768,44 +806,63 @@ KGet::~KGet()
 TransferHandler * KGet::createTransfer(const KUrl &src, const KUrl &dest, const QString& groupName, 
                           bool start, const QDomElement * e)
 {
-    kDebug(5001) << "srcUrl= " << src.url() << "  " 
-                             << "destUrl= " << dest.url() 
-                             << "group= _" << groupName << "_" << endl;
+    QList<TransferHandler*> transfer = createTransfers(QList<TransferData>() << TransferData(src, dest, groupName, start, e));
+    return (transfer.isEmpty() ? 0 : transfer.first());
+}
 
-    TransferGroup * group = m_transferTreeModel->findGroup(groupName);
-    if (group==0)
-    {
-        kDebug(5001) << "KGet::createTransfer  -> group not found";
-        group = m_transferTreeModel->transferGroups().first();
+QList<TransferHandler*> KGet::createTransfers(const QList<TransferData> &dataItems)
+{
+    QList<TransferHandler*> handlers;
+    if (dataItems.isEmpty()) {
+        return handlers;
     }
 
-    Transfer * newTransfer;
-    foreach (TransferFactory *factory, m_transferFactories)
-    {
-        kDebug(5001) << "Trying plugin   n.plugins=" << m_transferFactories.size();
-        if((newTransfer = factory->createTransfer(src, dest, group, m_scheduler, e)))
-        {
-//             kDebug(5001) << "KGet::createTransfer   ->   CREATING NEW TRANSFER ON GROUP: _" << group->name() << "_";
-            newTransfer->create();
-            newTransfer->load(e);
-            m_transferTreeModel->addTransfer(newTransfer, group);
-            
-            KGet::showNotification(m_mainWindow, "added",
-                                   i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", newTransfer->source().pathOrUrl()),
-                                   "kget", i18n("Download added"));
+    QList<bool> start;
+    QHash<TransferGroup*, QList<Transfer*> > groups;
 
-            if(start)
-                newTransfer->handler()->start();
+    foreach (const TransferData &data, dataItems) {
+        kDebug(5001) << "srcUrl=" << data.src << " destUrl=" << data.dest << " group=" << data.groupName;
 
-            return newTransfer->handler();
+        TransferGroup *group = m_transferTreeModel->findGroup(data.groupName);
+        if (!group) {
+            kDebug(5001) << "KGet::createTransfer  -> group not found";
+            group = m_transferTreeModel->transferGroups().first();
+        }
+
+        Transfer *newTransfer = 0;
+        foreach (TransferFactory *factory, m_transferFactories) {
+            kDebug(5001) << "Trying plugin   n.plugins=" << m_transferFactories.size();
+            if ((newTransfer = factory->createTransfer(data.src, data.dest, group, m_scheduler, data.e))) {
+    //             kDebug(5001) << "KGet::createTransfer   ->   CREATING NEW TRANSFER ON GROUP: _" << group->name() << "_";
+                newTransfer->create();
+                newTransfer->load(data.e);
+                handlers << newTransfer->handler();
+                groups[group] << newTransfer;
+                start << data.start;
+                break;
+            }
+        }
+        if (!newTransfer) {
+            KGet::showNotification(m_mainWindow, "error",
+                                   i18n("Unable to continue: KGet cannot download using this protocol."), "dialog-error", i18n("Protocol unsupported"));
+
+            kWarning(5001) << "Warning! No plugin found to handle" << data.src;
         }
     }
-    
-    KGet::showNotification(m_mainWindow, "error",
-                           i18n("Unable to continue: KGet cannot download using this protocol."), "dialog-error", i18n("Protocol unsupported"));
 
-    kDebug(5001) << "Warning! No plugin found to handle the given url";
-    return 0;
+    //add the created transfers to the model and start them if specified
+    QHash<TransferGroup*, QList<Transfer*> >::const_iterator it;
+    QHash<TransferGroup*, QList<Transfer*> >::const_iterator itEnd = groups.constEnd();
+    for (it = groups.constBegin(); it != itEnd; ++it) {
+        KGet::model()->addTransfers(it.value(), it.key());
+    }
+    for (int i = 0; i < handlers.count(); ++i) {
+        if (start[i]) {
+            handlers[i]->start();
+        }
+    }
+
+    return handlers;//TODO implement error message if it is 0, or should the addTransfers stuff do that, in case if the numer of returned items does not match the number of sent data?
 }
 
 TransferDataSource * KGet::createTransferDataSource(const KUrl &src, const QDomElement &type, QObject *parent)
@@ -1147,11 +1204,11 @@ GenericObserver::GenericObserver(QObject *parent)
     m_finishAction(0)
 {
     connect(KGet::model(), SIGNAL(groupRemovedEvent(TransferGroupHandler*)), SLOT(groupRemovedEvent(TransferGroupHandler*)));
-    connect(KGet::model(), SIGNAL(transferAddedEvent(TransferHandler*, TransferGroupHandler*)), 
-                           SLOT(transferAddedEvent(TransferHandler*, TransferGroupHandler*)));
+    connect(KGet::model(), SIGNAL(transfersAddedEvent(QList<TransferHandler*>)),
+                           SLOT(transfersAddedEvent(QList<TransferHandler*>)));
     connect(KGet::model(), SIGNAL(groupAddedEvent(TransferGroupHandler*)), SLOT(groupAddedEvent(TransferGroupHandler*)));
-    connect(KGet::model(), SIGNAL(transferRemovedEvent(TransferHandler*, TransferGroupHandler*)), 
-                           SLOT(transferRemovedEvent(TransferHandler*, TransferGroupHandler*)));
+    connect(KGet::model(), SIGNAL(transfersRemovedEvent(QList<TransferHandler*>)),
+                           SLOT(transfersRemovedEvent(QList<TransferHandler*>)));
     connect(KGet::model(), SIGNAL(transfersChangedEvent(QMap<TransferHandler*, Transfer::ChangesFlags>)), 
                            SLOT(transfersChangedEvent(QMap<TransferHandler*, Transfer::ChangesFlags>)));
     connect(KGet::model(), SIGNAL(groupsChangedEvent(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags>)), 
@@ -1176,19 +1233,17 @@ void GenericObserver::groupRemovedEvent(TransferGroupHandler *handler)
     KGet::save();
 }
 
-void GenericObserver::transferAddedEvent(TransferHandler *handler, TransferGroupHandler *group)
+void GenericObserver::transfersAddedEvent(const QList<TransferHandler*> &handlers)
 {
-    Q_UNUSED(handler)
-    Q_UNUSED(group)
+    Q_UNUSED(handlers)
     requestSave();
     KGet::calculateGlobalSpeedLimits();
     KGet::checkSystemTray();
 }
 
-void GenericObserver::transferRemovedEvent(TransferHandler *handler, TransferGroupHandler *group)
+void GenericObserver::transfersRemovedEvent(const QList<TransferHandler*> &handlers)
 {
-    Q_UNUSED(handler)
-    Q_UNUSED(group)
+    Q_UNUSED(handlers)
     requestSave();
     KGet::calculateGlobalSpeedLimits();
     KGet::checkSystemTray();
