@@ -10,6 +10,8 @@
 */
 #include "kgetkjobadapter.h"
 
+#include <KLocale>
+
 KGetKJobAdapter::KGetKJobAdapter(QObject *parent, TransferHandler *transfer) : KJob(parent)
 {
     m_transferHandler = transfer;
@@ -38,10 +40,11 @@ unsigned long KGetKJobAdapter::percent() const
 
 void KGetKJobAdapter::slotUpdateDescription()
 {
-    emit description(this, "KGet Transfer", 
+    emit description(this, i18n("KGet Transfer"), 
                     qMakePair(QString("source"), m_transferHandler->source().prettyUrl()),
                     qMakePair(QString("destination"), m_transferHandler->dest().prettyUrl()));
 
+    emitSpeed(m_transferHandler->downloadSpeed());
     setProcessedAmount(KJob::Bytes, processedAmount(KJob::Bytes));
     setTotalAmount(KJob::Bytes, totalAmount(KJob::Bytes));
     setPercent(percent());
