@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright (C) 2005 Dario Massarin <nekkar@libero.it>
+   Copyright (C) 2010 Matthias Fuchs <mat69@gmx.net>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -66,14 +67,16 @@ int TransferGroup::uploadSpeed()
 
 bool TransferGroup::supportsSpeedLimits()
 {
-    foreach (Job * job, runningJobs()) {
+    QList<Job*> jobs = runningJobs();
+    foreach (Job *job, jobs) {
         Transfer * transfer = static_cast<Transfer*>(job);
         if (!(transfer->capabilities() & Transfer::Cap_SpeedLimit)) {
             return false;
         }
     }
 
-    return true;
+    //empty jobs can't support a speed limit
+    return !jobs.isEmpty();
 }
 
 void TransferGroup::setStatus(Status queueStatus)
@@ -85,19 +88,23 @@ void TransferGroup::setStatus(Status queueStatus)
 
 void TransferGroup::append(Transfer * transfer)
 {
-    kDebug(5001) << "TransferGroup::append";
-
-    Transfer * after;
-    if(size() == 0) 
-        after = 0;
-    else
-        after = static_cast<Transfer *> (last());
-
     JobQueue::append(transfer);
 
     calculateSpeedLimits();
 }
 
+void TransferGroup::append(const QList<Transfer*> &transfers)
+{
+    QList<Job*> jobs;
+    foreach (Transfer *transfer, transfers) {
+        jobs << transfer;
+    }
+    JobQueue::append(jobs);
+
+    calculateSpeedLimits();
+}
+
+
 void TransferGroup::prepend(Transfer * transfer)
 {
     JobQueue::prepend(transfer);
@@ -119,6 +126,17 @@ void TransferGroup::remove(Transfer * transfer)
     calculateSpeedLimits();
 }
 
+void TransferGroup::remove(const QList<Transfer*> &transfers)
+{
+    QList<Job*> jobs;
+    foreach (Transfer *transfer, transfers) {
+        jobs << transfer;
+    }
+    JobQueue::remove(jobs);
+
+    calculateSpeedLimits();
+}
+
 void TransferGroup::move(Transfer * transfer, Transfer * after)
 {
     if(transfer == after)
@@ -221,11 +239,11 @@ void TransferGroup::calculateDownloadLimit()
     kDebug(5001) << "Calculate new DownloadLimit of " + QString::number(m_downloadLimit);
     if (supportsSpeedLimits())
     {
-        int n = runningJobs().count();
+        const QList<Job*> running = runningJobs();
+        int n = running.count();
         int pool = 0;//We create a pool where we have some KiB/s to go to other transfer's...
         QList<Transfer*> transfersNeedSpeed;
-        foreach (Job * job, runningJobs())
-        {
+        foreach (Job *job, running) {
             Transfer * transfer = static_cast<Transfer*>(job);
             if (transfer)
             {
@@ -264,11 +282,11 @@ void TransferGroup::calculateUploadLimit()
     kDebug(5001) << "Calculate new Upload Limit of " + QString::number(m_uploadLimit);
     if (supportsSpeedLimits())
     {
-        int n = runningJobs().count();
+        const QList<Job*> running = runningJobs();
+        int n = running.count();
         int pool = 0;//We create a pool where we have some KiB/s to go to other transfer's...
         QList<Transfer*> transfersNeedSpeed;
-        foreach (Job * job, runningJobs())
-        {
+        foreach (Job *job, running) {
             Transfer * transfer = static_cast<Transfer*>(job);
             if (transfer)
             {
@@ -372,9 +390,11 @@ void TransferGroup::load(const QDomElement & e)
     QDomNodeList nodeList = e.elementsByTagName("Transfer");
     int nItems = nodeList.length();
 
-    for(int i=0; i<nItems; i++)
-    {
-        kDebug(5001) << "TransferGroup::load -> addTransfer";
-        KGet::addTransfer( nodeList.item(i).toElement(), name() );
+    QList<QDomElement> elements;
+    for (int i = 0; i < nItems; ++i) {
+        elements << nodeList.item(i).toElement();
     }
+
+    kDebug(5001) << "TransferGroup::load ->" << "add" << nItems << "transfers";
+    KGet::addTransfers(elements, name());
 }
