@@ -24,8 +24,7 @@
 
 #include <kdebug.h>
 #include <klocale.h>
-#include <kio/netaccess.h>
-#include <kiconloader.h>
+#include <KMimeType>
 
 #include <qmimedata.h>
 
@@ -83,8 +82,15 @@ QVariant TransferModelItem::data(int role) const
     {
         switch (column())
         {
-            case 0:
-                return KIO::pixmapForUrl(m_transferHandler->dest(), 0, KIconLoader::Desktop, 16);
+            case 0: {
+                //store the icon for speed improvements, KIconLoader should make sure, that
+                //the icon data gets shared
+                if (m_mimeType.isNull()) {
+                    m_mimeType = KIcon(KMimeType::iconNameForUrl(m_transferHandler->dest().url()));
+                }
+
+                return m_mimeType;
+            }
             case 1:
                 return m_transferHandler->statusPixmap();
         }
@@ -194,11 +200,20 @@ void TransferTreeModel::addGroup(TransferGroup * group)
 
 void TransferTreeModel::delGroup(TransferGroup * group)
 {
-    GroupModelItem * item = itemFromTransferGroupHandler(group->handler());
-    if (item)
-        takeRow(item->row());
-    else
+    GroupModelItem *item = itemFromTransferGroupHandler(group->handler());
+    if (!item) {
         return;
+    }
+
+    QList<Transfer*> transfers;
+    JobQueue::iterator it;
+    JobQueue::iterator itEnd = group->end();
+    for (it = group->begin(); it != itEnd; ++it) {
+        transfers << static_cast<Transfer*>(*it);
+    }
+    delTransfers(transfers);
+
+    takeRow(item->row());
 
     m_changedGroups.removeAll(group->handler());
 
@@ -209,46 +224,120 @@ void TransferTreeModel::delGroup(TransferGroup * group)
     m_transferGroups.removeAll(item);
 }
 
-void TransferTreeModel::addTransfer(Transfer * transfer, TransferGroup * group)
+void TransferTreeModel::addTransfers(const QList<Transfer*> &transfers, TransferGroup *group)
 {
-    group->append(transfer);
+    ModelItem *parentItem = itemFromTransferGroupHandler(group->handler());
+    const QModelIndex parentIndex = parentItem->index();
+    beginInsertRows(parentItem->index(), parentItem->rowCount(), parentItem->rowCount() + transfers.count() - 1);
 
-    QList<QStandardItem*> items;
-    for (int i = 0; i != transfer->handler()->columnCount(); i++)
-        items << new TransferModelItem(transfer->handler());
+    //HACK blocks all signals from the model when adding multiple items,
+    //that way rowsInserted gets only emitted once, and not constantly when doing appendRow
+    //change this once there is a better way to append many transfers at once
+    blockSignals(true);
 
-    itemFromTransferGroupHandler(group->handler())->appendRow(items);
+    //now create and add the new items
+    QList<TransferHandler*> handlers;
+    group->append(transfers);
+    foreach (Transfer *transfer, transfers) {
+        TransferHandler *handler = transfer->handler();
+        handlers << handler;
 
-    emit transferAddedEvent(transfer->handler(), group->handler());
-    
-    m_transfers.append(static_cast<TransferModelItem*>(items.first()));
+        QList<QStandardItem*> items;
+        for (int i = 0; i != handler->columnCount(); ++i) {
+            items << new TransferModelItem(handler);
+        }
 
-    DBusTransferWrapper * wrapper = new DBusTransferWrapper(transfer->handler());
-    new TransferAdaptor(wrapper);
-    QDBusConnection::sessionBus().registerObject(transfer->handler()->dBusObjectPath(), wrapper);
+        parentItem->appendRow(items);
+
+        m_transfers.append(static_cast<TransferModelItem*>(items.first()));
+
+        DBusTransferWrapper * wrapper = new DBusTransferWrapper(handler);
+        new TransferAdaptor(wrapper);
+        QDBusConnection::sessionBus().registerObject(handler->dBusObjectPath(), wrapper);
+    }
+
+    //notify the rest of the changes
+    blockSignals(false);
+    endInsertRows();
+    emit transfersAddedEvent(handlers);
 }
 
-void TransferTreeModel::delTransfer(Transfer * transfer)
+void TransferTreeModel::delTransfers(const QList<Transfer*> &t)
 {
-    TransferModelItem * item = itemFromTransferHandler(transfer->handler());
+    QList<Transfer*> transfers = t;
+    QList<TransferHandler*> handlers;
 
-    if (!item)
-        return;
+    //find all valid items and sort them according to their groups
+    QHash<TransferGroup*, QList<TransferModelItem*> > groups;
+    QHash<TransferGroup*, QList<Transfer*> > groupsTransfer;
+    {
+        QList<Transfer*>::iterator it;
+        QList<Transfer*>::iterator itEnd = transfers.end();
+        for (it = transfers.begin(); it != itEnd; ) { 
+            TransferModelItem *item = itemFromTransferHandler((*it)->handler());
+            if (item) {
+                handlers << (*it)->handler();
+                groups[(*it)->group()] << item;
+                groupsTransfer[(*it)->group()] << *it;
+                ++it;
+            } else {
+                it = transfers.erase(it);
+            }
+        }
+    }
 
-    emit transferAboutToBeRemovedEvent(transfer->handler(), transfer->group()->handler());
-    
-    item->parent()->takeRow(item->row());
+    emit transfersAboutToBeRemovedEvent(handlers);
 
-    QDBusConnection::sessionBus().unregisterObject(transfer->handler()->dBusObjectPath());
+    //remove the items from the model
+    {
+        QHash<TransferGroup*, QList<TransferModelItem*> >::iterator it;
+        QHash<TransferGroup*, QList<TransferModelItem*> >::iterator itEnd = groups.end();
+        for (it = groups.begin(); it != itEnd; ++it) {
+            const int numItems = (*it).count();
+            QStandardItem *parentItem = (*it).first()->parent();
+            QModelIndex parentIndex = parentItem->index();
+            if (numItems == parentItem->rowCount()) {
+                for (int i = 0; i < numItems; ++i) {
+                    m_transfers.removeAll((*it)[i]);
+                }
+                removeRows(0, numItems, parentIndex);
+                continue;
+            }
 
-    TransferGroup * group = transfer->group();
+            int rowStart = (*it).first()->row();
+            int numRows = 1;
+            m_transfers.removeAll((*it).first());
+            for (int i = 1; i < numItems; ++i) {
+                //previous item is neighbour
+                if (rowStart + numRows == (*it)[i]->row()) {
+                    ++numRows;
+                //no neighbour, so start again
+                } else {
+                    removeRows(rowStart, numRows, parentIndex);
+                    rowStart = (*it)[i]->row();
+                    numRows = 1;
+                }
+                m_transfers.removeAll((*it)[i]);
+            }
+            //remove last items
+            removeRows(rowStart, numRows, parentIndex);
+        }
+    }
 
-    group->remove(transfer);
-    m_changedTransfers.removeAll(transfer->handler());
+    foreach(Transfer *transfer, transfers) {
+        QDBusConnection::sessionBus().unregisterObject(transfer->handler()->dBusObjectPath());
+        m_changedTransfers.removeAll(transfer->handler());
+    }
 
-    m_transfers.removeAll(item);
+    {
+        QHash<TransferGroup*, QList<Transfer*> >::iterator it;
+        QHash<TransferGroup*, QList<Transfer*> >::iterator itEnd = groupsTransfer.end();
+        for (it = groupsTransfer.begin(); it != itEnd; ++it) {
+            it.key()->remove(it.value());
+        }
+    }
 
-    emit transferRemovedEvent(transfer->handler(), group->handler());
+    emit transfersRemovedEvent(handlers);
 }
 
 TransferModelItem * TransferTreeModel::itemFromTransferHandler(TransferHandler * handler)
