@@ -401,8 +401,8 @@ void MainWindow::slotDelayedInit()
         KGet::setGlobalUploadLimit(0);
     }
 
-    connect(KGet::model(), SIGNAL(transferAddedEvent(TransferHandler *, TransferGroupHandler *)), this, SLOT(slotUpdateTitlePercent()));
-    connect(KGet::model(), SIGNAL(transferRemovedEvent(TransferHandler *, TransferGroupHandler *)), this, SLOT(slotUpdateTitlePercent()));
+    connect(KGet::model(), SIGNAL(transfersAddedEvent(QList<TransferHandler*>)), this, SLOT(slotUpdateTitlePercent()));
+    connect(KGet::model(), SIGNAL(transfersRemovedEvent(QList<TransferHandler*>)), this, SLOT(slotUpdateTitlePercent()));
     connect(KGet::model(), SIGNAL(transfersChangedEvent(QMap<TransferHandler *, Transfer::ChangesFlags>)), 
                            SLOT(slotTransfersChanged(QMap<TransferHandler *, Transfer::ChangesFlags>)));
     connect(KGet::model(), SIGNAL(groupsChangedEvent(QMap<TransferGroupHandler *, TransferGroup::ChangesFlags>)),
@@ -596,8 +596,11 @@ void MainWindow::slotStartAllDownload()
 
 void MainWindow::slotStartSelectedDownload()
 {
-    foreach(TransferHandler * it, KGet::selectedTransfers())
-        it->start();
+    KGet::setSuspendScheduler(true);
+    foreach (TransferHandler *transfer, KGet::selectedTransfers()) {
+        transfer->start();
+    }
+    KGet::setSuspendScheduler(false);
 }
 
 void MainWindow::slotStopDownload()
@@ -620,8 +623,11 @@ void MainWindow::slotStopAllDownload()
 
 void MainWindow::slotStopSelectedDownload()
 {
-    foreach (TransferHandler * it, KGet::selectedTransfers())
-        it->stop();
+    KGet::setSuspendScheduler(true);
+    foreach (TransferHandler *transfer, KGet::selectedTransfers()) {
+        transfer->stop();
+    }
+    KGet::setSuspendScheduler(false);
 }
 
 void MainWindow::slotDeleteSelected()
@@ -641,12 +647,10 @@ void MainWindow::slotDeleteSelected()
         }
     }
 
-    foreach (TransferHandler * it, KGet::selectedTransfers())
-    {
-        it->stop();
-        m_viewsContainer->closeTransferDetails(it);
-        KGet::delTransfer(it);
+    foreach (TransferHandler * it, KGet::selectedTransfers()) {
+        m_viewsContainer->closeTransferDetails(it);//TODO make it take QList?
     }
+    KGet::delTransfers(KGet::selectedTransfers());
 }
 
 void MainWindow::slotRedownloadSelected()
@@ -700,12 +704,10 @@ void MainWindow::slotTransfersCopySourceUrl()
 
 void MainWindow::slotDeleteFinished()
 {
-    foreach(TransferHandler * it, KGet::finishedTransfers())
-    {
-        it->stop();
+    foreach(TransferHandler * it, KGet::finishedTransfers()) {
         m_viewsContainer->closeTransferDetails(it);
-        KGet::delTransfer(it);
     }
+    KGet::delTransfers(KGet::finishedTransfers());
 }
 
 void MainWindow::slotConfigureNotifications()
