@@ -31,7 +31,8 @@
 #include <util/log.h>
 #include <peer/authenticationmonitor.h>
 #include <interfaces/trackerinterface.h>
-#include <btversion.h>
+#include <utp/utpserver.h>
+#include <version.h>
 
 #include <KDebug>
 #include <KLocale>
@@ -70,6 +71,8 @@ BTTransfer::BTTransfer(TransferGroup* parent, TransferFactory* factory,
 #endif
 {
     m_directory = m_dest.upUrl();//FIXME test
+
+    setCapabilities(Transfer::Cap_Moving | Transfer::Cap_Renaming | Transfer::Cap_Resuming | Transfer::Cap_SpeedLimit);
 }
 
 BTTransfer::~BTTransfer()
@@ -116,11 +119,6 @@ void BTTransfer::deinit()
 }
 
 /** Reimplemented functions from Transfer-Class **/
-bool BTTransfer::isResumable() const
-{
-    return true;
-}
-
 bool BTTransfer::isStalled() const
 {
     return (status() == Job::Running) && (downloadSpeed() == 0) && torrent && torrent->getStats().status == bt::STALLED;
@@ -166,7 +164,7 @@ bool BTTransfer::setDirectory(const KUrl &newDirectory)
     //check if the newDestination is the same as the old
     KUrl temp = newDirectory;
     temp.addPath(torrent->getStats().torrent_name);
-    if (isResumable() && newDirectory.isValid() && (newDirectory != dest()) && (temp != dest()))
+    if (newDirectory.isValid() && (newDirectory != dest()) && (temp != dest()))
     {
         if (torrent->changeOutputDir(newDirectory.pathOrUrl(), bt::TorrentInterface::MOVE_FILES))
         {
@@ -200,9 +198,7 @@ void BTTransfer::newDestResult()
 void BTTransfer::stop()
 {
     if (m_movingFile)
-    {
         return;
-    }
 
     if (m_ready)
     {
@@ -235,7 +231,7 @@ void BTTransfer::load(const QDomElement *element)
 {
     Transfer::load(element);
 
-    if((m_totalSize == m_downloadedSize) && (m_totalSize != 0))
+    if ((m_totalSize == m_downloadedSize) && (m_totalSize != 0))
     {
         setStatus(Job::Stopped, i18nc("transfer state: finished", "Finished"), SmallIcon("dialog-ok"));
     }
@@ -254,7 +250,9 @@ void BTTransfer::load(const QDomElement *element)
 
 void BTTransfer::setPort(int port)
 {
-    bt::Globals::instance().getServer().changePort(port);
+    bt::Globals::instance().getTCPServer().changePort(port);
+    if (BittorrentSettings::enableUTP())
+        bt::Globals::instance().getUTPServer().changePort(port + 1);
 }
 
 void BTTransfer::setSpeedLimits(int ulLimit, int dlLimit)
@@ -269,16 +267,14 @@ void BTTransfer::setSpeedLimits(int ulLimit, int dlLimit)
 void BTTransfer::addTracker(const QString &url)
 {
     kDebug(5001);
-    if(torrent->getStats().priv_torrent)
-    {
-	KMessageBox::sorry(0, i18n("Cannot add a tracker to a private torrent."));
-	return;
+    if(torrent->getStats().priv_torrent) {
+        KMessageBox::sorry(0, i18n("Cannot add a tracker to a private torrent."));
+        return;
     }
 
-    if(!KUrl(url).isValid())
-    {
-	KMessageBox::error(0, i18n("Malformed URL."));
-	return;
+    if(!KUrl(url).isValid()) {
+       KMessageBox::error(0, i18n("Malformed URL."));
+       return;
     }
 
     torrent->getTrackersList()->addTracker(url,true);
@@ -449,16 +445,17 @@ void BTTransfer::btTransferInit(const KUrl &src, const QByteArray &data)
     bt::SetClientInfo("KGet", 2, KDE_VERSION_MINOR, KDE_VERSION_RELEASE, bt::NORMAL, "KG");//Set client info to KGet
 
     bt::Uint16 i = 0;
-    do
-    {
-        bt::Globals::instance().initServer(BittorrentSettings::port() + i);
+    while (!bt::Globals::instance().initTCPServer(BittorrentSettings::port() + i) && i < 10)
         i++;
-    }while (!bt::Globals::instance().getServer().isOK() && i < 10);
-
-    if (!bt::Globals::instance().getServer().isOK()) {
+    
+    if (i == 10) {
         setStatus(Job::Aborted, i18n("An error occurred...."), SmallIcon("document-preview"));
         return;
     }
+    if (BittorrentSettings::enableUTP()) {
+        while (!bt::Globals::instance().initUTPServer(BittorrentSettings::port() + i) && i < 10) //We don't care if it fails for now as UTP is experimental...
+            i++;
+    }
 
     QDir tmpDir(m_tmp + m_source.fileName().remove(".torrent"));
     if (tmpDir.exists())
@@ -496,6 +493,8 @@ void BTTransfer::btTransferInit(const KUrl &src, const QByteArray &data)
     catch (bt::Error &err)
     {
         m_ready = false;
+        torrent->deleteLater();
+        torrent = 0;
         setStatus(Job::Aborted, i18n("An error occurred...."), SmallIcon("document-preview"));
         KMessageBox::error(0, err.toString(), i18n("Error"));
     }
@@ -750,7 +749,7 @@ void BTTransfer::filesSelected()
 //     setTransferChange(Tc_TotalSize | Tc_DownloadedSize | Tc_Percent, true);
 }
 
-FileModel *BTTransfer::fileModel()//TODO korrektes file-model wenn nur eine datei im torrent!
+FileModel *BTTransfer::fileModel()//TODO correct file model for one-file-torrents
 {
     if (!m_fileModel)
     {
