@@ -27,39 +27,14 @@
 #include <util/log.h>
 #include <torrent/torrent.h>
 #include <diskio/chunkmanager.h>
-#include <interfaces/chunkdownloadinterface.h>
+#include <diskio/piecedata.h>
+#include <net/socketmonitor.h>
 #include "httpconnection.h"
 
 namespace bt
 {
-	class WebSeedChunkDownload : public ChunkDownloadInterface
-	{
-	public:
-		WebSeedChunkDownload(WebSeed* ws,const QString & url,Uint32 index,Uint32 total) 
-			: ws(ws),url(url),chunk(index),total_pieces(total),pieces_downloaded(0)
-		{}
-		 
-		virtual ~WebSeedChunkDownload()
-		{
-		}
-		
-		void getStats(Stats & s)
-		{
-			s.current_peer_id = url;
-			s.chunk_index = chunk;
-			s.num_downloaders = 1;
-			s.download_speed = ws->getDownloadRate();
-			s.pieces_downloaded = pieces_downloaded;
-			s.total_pieces = total_pieces;
-		}
-		
-		WebSeed* ws;
-		QString url;
-		Uint32 chunk;
-		Uint32 total_pieces;
-		Uint32 pieces_downloaded;
-	};
 	
+			
 	QString WebSeed::proxy_host;
 	Uint16 WebSeed::proxy_port = 8080;
 	bool WebSeed::proxy_enabled = false;
@@ -103,9 +78,12 @@ namespace bt
 	
 	void WebSeed::reset()
 	{
+		if (current)
+			chunkStopped();
+		
 		if (conn)
 		{
-			delete conn;
+			conn->deleteLater();
 			conn = 0;
 		}
 		
@@ -113,6 +91,12 @@ namespace bt
 		num_failures = 0;
 		status = i18n("Not connected");
 	}
+	
+	void WebSeed::cancel() 
+	{
+		reset();
+	}
+
 
 	bool WebSeed::busy() const
 	{
@@ -126,19 +110,40 @@ namespace bt
 		else
 			return 0;
 	}
+	
+	void WebSeed::connectToServer()
+	{
+		KUrl dst = url;
+		if (redirected_url.isValid())
+			dst = redirected_url;
+		
+		if (!proxy_enabled)
+		{
+			QString proxy = KProtocolManager::proxyForUrl(dst); // Use KDE settings
+			if (proxy.isNull() || proxy == "DIRECT")
+				conn->connectTo(dst); // direct connection 
+			else
+			{
+				KUrl proxy_url(proxy);
+				conn->connectToProxy(proxy_url.host(),proxy_url.port() <= 0 ? 80 : proxy_url.port());
+			}
+		}
+		else 
+		{
+			if (proxy_host.isNull())
+				conn->connectTo(dst); // direct connection 
+			else
+				conn->connectToProxy(proxy_host,proxy_port); // via a proxy
+		}
+		status = conn->getStatusString();
+	}
 		
 	void WebSeed::download(Uint32 first,Uint32 last)
 	{
-		Out(SYS_CON|LOG_DEBUG) << "WebSeed::download " << first << "-" << last << endl;
-		first_chunk = first;
-		last_chunk = last;
-		cur_chunk = first;
-		bytes_of_cur_chunk = 0;
-		
-		QString path = url.path();
-		if (path.endsWith('/'))
-			path += tor.getNameSuggestion();
+		if (!enabled)
+			return;
 		
+		Out(SYS_CON|LOG_DEBUG) << "WebSeed: downloading " << first << "-" << last << " from " << url.prettyUrl() << endl;
 		// open connection and connect if needed
 		if (!conn)
 		{
@@ -148,55 +153,103 @@ namespace bt
 		
 		if (!conn->connected())
 		{
-			if (!proxy_enabled)
-			{
-				QString proxy = KProtocolManager::proxyForUrl(url); // Use KDE settings
-				if (proxy.isNull() || proxy == "DIRECT")
-					conn->connectTo(url); // direct connection 
-				else
-				{
-					KUrl proxy_url(proxy);
-					conn->connectToProxy(proxy_url.host(),proxy_url.port() <= 0 ? 80 : proxy_url.port());
-				}
-			}
-			else 
-			{
-				if (proxy_host.isNull())
-					conn->connectTo(url); // direct connection 
-				else
-					conn->connectToProxy(proxy_host,proxy_port); // via a proxy
-			}
-			status = conn->getStatusString();
+			connectToServer();
 		}
 		
+		if (first == cur_chunk && last == last_chunk && bytes_of_cur_chunk > 0)
+		{
+			// we already have some of the data, so reuse it
+			continueCurChunk();
+			return;
+		}
+		
+		first_chunk = first;
+		last_chunk = last;
+		cur_chunk = first;
+		bytes_of_cur_chunk = 0;
+		
+		QString path = url.path();
+		if (path.endsWith('/') && !isUserCreated())
+			path += tor.getNameSuggestion();
+		
 		if (tor.isMultiFile())
 		{
+			range_queue.clear();
 			// make the list of ranges to download
-			QList<Range> ranges;
 			for (Uint32 i = first_chunk;i <= last_chunk;i++)
 			{
-				doChunk(i,ranges);
+				fillRangeList(i);
 			}
 			
-			// add a request for each range
-			foreach (const Range & r,ranges)
+			if (range_queue.count() > 0)
 			{
+				// send the first request
+				Range r = range_queue[0];
+				range_queue.pop_front();
 				const TorrentFile & tf = tor.getFile(r.file);
-				conn->get(url.host(),path + '/' + tf.getPath(),r.off,r.len);
+				QString host = redirected_url.isValid() ? redirected_url.host() : url.host();
+				conn->get(host,path + '/' + tf.getPath(),r.off,r.len);
 			}
 		}
 		else
 		{
 			Uint64 len = (last_chunk - first_chunk) * tor.getChunkSize();
 			// last chunk can have a different size
-			if (last_chunk == tor.getNumChunks() - 1 && tor.getFileLength() % tor.getChunkSize() > 0)
-				len += tor.getFileLength() % tor.getChunkSize();
+			if (last_chunk == tor.getNumChunks() - 1)
+				len += tor.getLastChunkSize();
 			else
 				len += tor.getChunkSize(); 
 			
-			conn->get(url.host(),path,first_chunk * tor.getChunkSize(),len);
+			QString host = redirected_url.isValid() ? redirected_url.host() : url.host();
+			conn->get(host,path,first_chunk * tor.getChunkSize(),len);
 		}
+	}
+	
+	void WebSeed::continueCurChunk()
+	{
+		QString path = url.path();
+		if (path.endsWith('/') && !isUserCreated())
+			path += tor.getNameSuggestion();
 		
+		Out(SYS_GEN|LOG_DEBUG) << "WebSeed: continuing current chunk " << cur_chunk << " " << bytes_of_cur_chunk << endl;
+		first_chunk = cur_chunk;
+		if (tor.isMultiFile())
+		{
+			range_queue.clear();
+			// make the list of ranges to download
+			for (Uint32 i = first_chunk;i <= last_chunk;i++)
+			{
+				fillRangeList(i);
+			}
+			
+			bt::Uint32 length = 0;
+			while (range_queue.count() > 0)
+			{
+				// send the first request, but skip the data we already have
+				Range r = range_queue[0];
+				range_queue.pop_front();
+				if (length >= bytes_of_cur_chunk)
+				{
+					const TorrentFile & tf = tor.getFile(r.file);
+					QString host = redirected_url.isValid() ? redirected_url.host() : url.host();
+					conn->get(host,path + '/' + tf.getPath(),r.off,r.len);
+					break;
+				}
+				length += r.len;
+			}
+		}
+		else
+		{
+			Uint64 len = (last_chunk - first_chunk) * tor.getChunkSize();
+			// last chunk can have a different size
+			if (last_chunk == tor.getNumChunks() - 1)
+				len += tor.getLastChunkSize();
+			else
+				len += tor.getChunkSize(); 
+			
+			QString host = redirected_url.isValid() ? redirected_url.host() : url.host();
+			conn->get(host,path,first_chunk * tor.getChunkSize() + bytes_of_cur_chunk,len - bytes_of_cur_chunk);
+		}
 		chunkStarted(cur_chunk);
 	}
 	
@@ -206,28 +259,29 @@ namespace bt
 		Uint32 pieces_count = csize / MAX_PIECE_LEN;
 		if (csize % MAX_PIECE_LEN > 0)
 			pieces_count++;
-		current = new WebSeedChunkDownload(this,url.prettyUrl(),chunk,pieces_count);
-		chunkDownloadStarted(current);
+	
+		if (!current)
+		{
+			current = new WebSeedChunkDownload(this,url.prettyUrl(),chunk,pieces_count);
+			chunkDownloadStarted(current,chunk);
+		}
+		else if (current->chunk != chunk)
+		{
+			chunkStopped();
+			current = new WebSeedChunkDownload(this,url.prettyUrl(),chunk,pieces_count);
+			chunkDownloadStarted(current,chunk);
+		}
 	}
 	
 	void WebSeed::chunkStopped()
 	{
 		if (current)
 		{
-			chunkDownloadFinished(current);
+			chunkDownloadFinished(current,current->chunk);
 			delete current;
 			current = 0;
 		}
 	}
-	
-	void WebSeed::retry()
-	{
-		if (!busy())
-			return;
-		
-		// lets try this again
-		download(cur_chunk,last_chunk);
-	}
 		
 	Uint32 WebSeed::update()
 	{
@@ -236,51 +290,72 @@ namespace bt
 		
 		if (!conn->ok())
 		{
+			readData();
 			Out(SYS_CON|LOG_DEBUG) << "WebSeed: connection not OK" << endl;
 			// shit happened delete connection
 			status = conn->getStatusString();
+			if (conn->responseCode() == 404)
+			{
+				// if not found then retire this webseed
+				num_failures = 3;
+				status = i18n("Not in use");
+			}
 			delete conn;
 			conn = 0;
-			
-			
 			chunkStopped();
-			
+			first_chunk = last_chunk = cur_chunk = tor.getNumChunks() + 1;
 			num_failures++;
-			if (num_failures < 3) // try again in 10 seconds
-				QTimer::singleShot(10*1000,this,SLOT(retry()));
-			else // try again but with a longer interval (2 minutes)
-				QTimer::singleShot(120*1000,this,SLOT(retry()));
-	
 			return 0;
 		}
 		else if (conn->closed())
 		{
+			// Make sure we handle all data
+			readData();
+			
 			Out(SYS_CON|LOG_DEBUG) << "WebSeed: connection closed" << endl;
 			delete conn;
 			conn = 0;
 			
 			status = i18n("Connection closed");
 			chunkStopped();
-			// lets try this again
-			download(cur_chunk,last_chunk);
-			status = conn->getStatusString();
-		}
-		else
-		{
-			QByteArray tmp;
-			while (conn->getData(tmp) && cur_chunk <= last_chunk)
+			if (last_chunk < tor.getNumChunks())
 			{
-				//Out(SYS_CON|LOG_DEBUG) << "WebSeed: handleData " << tmp.size() << endl;
-				handleData(tmp);
-				tmp.clear();
+				// lets try this again if we have not yet got the full range
+				download(cur_chunk,last_chunk);
+				status = conn->getStatusString();
 			}
-			
-			if (cur_chunk > last_chunk)
+		}
+		else if (conn->isRedirected())
+		{
+			// Make sure we handle all data
+			readData();
+			redirected(conn->redirectedUrl());
+		}
+		else 
+		{
+			readData();
+			if (range_queue.count() > 0 && conn->ready())
 			{
-				// if the current chunk moves past the last chunk, we are done
-				first_chunk = last_chunk = tor.getNumChunks() + 1;
-				num_failures = 0;
-				finished();
+				if (conn->closed())
+				{
+					// after a redirect it is possible that the connection is closed
+					// so we need to reconnect to the old url
+					conn->deleteLater();
+					conn = new HttpConnection();
+					conn->setGroupIDs(up_gid,down_gid);
+					connectToServer();
+				}
+				
+				QString path = url.path();
+				if (path.endsWith('/'))
+					path += tor.getNameSuggestion();
+				
+				// ask for the next range
+				Range r = range_queue[0];
+				range_queue.pop_front();
+				const TorrentFile & tf = tor.getFile(r.file);
+				QString host = redirected_url.isValid() ? redirected_url.host() : url.host();
+				conn->get(host,path + '/' + tf.getPath(),r.off,r.len);
 			}
 			status = conn->getStatusString();
 		}
@@ -291,8 +366,30 @@ namespace bt
 		return ret;
 	}
 	
+	void WebSeed::readData()
+	{
+		QByteArray tmp;
+		while (conn->getData(tmp) && cur_chunk <= last_chunk)
+		{
+			//Out(SYS_CON|LOG_DEBUG) << "WebSeed: handleData " << tmp.size() << endl;
+			if (!current)
+				chunkStarted(cur_chunk);
+			handleData(tmp);
+			tmp.clear();
+		}
+		
+		if (cur_chunk > last_chunk)
+		{
+			// if the current chunk moves past the last chunk, we are done
+			first_chunk = last_chunk = tor.getNumChunks() + 1;
+			num_failures = 0;
+			finished();
+		}
+	}
+	
 	void WebSeed::handleData(const QByteArray & tmp)
 	{
+//		Out(SYS_GEN|LOG_DEBUG) << "Handling data: " << tmp.length() << " bytes" << endl;
 		Uint32 off = 0;
 		while (off < (Uint32)tmp.size() && cur_chunk <= last_chunk)
 		{
@@ -301,32 +398,42 @@ namespace bt
 			if (bl > tmp.size() - off)
 				bl = tmp.size() - off;
 					
-			if (c->getStatus() == Chunk::BUFFERED || c->getStatus() == Chunk::MMAPPED)
+			// ignore data if we already have it
+			if (c->getStatus() != Chunk::ON_DISK)
 			{
-				// only write when we have the chunk in memory
-				// if we already have the chunk we will then just ignore the data
-				memcpy(c->getData() + bytes_of_cur_chunk,tmp.data() + off,bl);
+				PieceData* p = c->getPiece(0,c->getSize(),false); 
+				if (p)
+					memcpy(p->data() + bytes_of_cur_chunk,tmp.data() + off,bl);
+				
 				downloaded += bl;
 			}
 			off += bl;
 			bytes_of_cur_chunk += bl;
 			current->pieces_downloaded = bytes_of_cur_chunk / MAX_PIECE_LEN;
+			
 			if (bytes_of_cur_chunk == c->getSize())
 			{
 				// we have one ready
 				bytes_of_cur_chunk = 0;
 				cur_chunk++;
-				if (c->getStatus() == Chunk::BUFFERED || c->getStatus() == Chunk::MMAPPED)
+				if (c->getStatus() != Chunk::ON_DISK)
+				{
 					chunkReady(c);
+				}
 				
 				chunkStopped();
 				if (cur_chunk <= last_chunk)
+				{
+					PieceData* p = cman.getChunk(cur_chunk)->getPiece(0,c->getSize(),false);
+					if (p)
+						p->ref(); // reference will be released when chunk is ready
 					chunkStarted(cur_chunk);
+				}
 			}
 		}
 	}
 
-	void WebSeed::doChunk(Uint32 chunk,QList<Range> & ranges)
+	void WebSeed::fillRangeList(Uint32 chunk)
 	{
 		QList<Uint32> tflist;
 		tor.calcChunkPos(chunk,tflist);
@@ -350,15 +457,15 @@ namespace bt
 				r.len = tf.getSize();
 			
 			// add the range
-			if (ranges.count() == 0)
-				ranges.append(r);
-			else if (ranges.back().file != r.file)
-				ranges.append(r);
+			if (range_queue.count() == 0)
+				range_queue.append(r);
+			else if (range_queue.back().file != r.file)
+				range_queue.append(r);
 			else
 			{
 				// the last range and this one are in the same file
 				// so expand it
-				Range & l = ranges.back();
+				Range & l = range_queue.back();
 				l.len += r.len;
 			}
 			
@@ -369,7 +476,68 @@ namespace bt
 	void WebSeed::onExcluded(Uint32 from,Uint32 to)
 	{
 		if (from <= first_chunk && first_chunk <= to && from <= last_chunk && last_chunk <= to)
+		{
 			reset();
+		}
 	}
+	
+	void WebSeed::chunkDownloaded(Uint32 chunk)
+	{
+		// reset if chunk downloaded is in the range we are currently downloading
+		if (chunk >= cur_chunk) 
+		{
+			reset();
+		}
+	}
+	
+	void WebSeed::setEnabled(bool on)
+	{
+		WebSeedInterface::setEnabled(on);
+		if (!on)
+		{
+			reset();
+		}
+	}
+	
+	void WebSeed::redirected(const KUrl & to_url)
+	{
+		delete conn;
+		conn = 0;
+		if (to_url.isValid() && to_url.protocol() == "http")
+		{
+			redirected_url = to_url;
+			download(cur_chunk,last_chunk);
+			status = conn->getStatusString();
+		}
+		else
+		{
+			num_failures = 3;
+			status = i18n("Not in use");
+			cur_chunk = last_chunk = first_chunk = tor.getNumChunks() + 1;
+		}
+	}
+
+	
+	////////////////////////////////////////////
+	
+	WebSeedChunkDownload::WebSeedChunkDownload(WebSeed* ws,const QString & url,Uint32 index,Uint32 total) 
+	: ws(ws),url(url),chunk(index),total_pieces(total),pieces_downloaded(0)
+	{}
+		 
+	WebSeedChunkDownload::~WebSeedChunkDownload()
+	{
+	}
+		
+	void WebSeedChunkDownload::getStats(Stats & s)
+	{
+		s.current_peer_id = url;
+		s.chunk_index = chunk;
+		s.num_downloaders = 1;
+		s.download_speed = ws->getDownloadRate();
+		s.pieces_downloaded = pieces_downloaded;
+		s.total_pieces = total_pieces;
+	}
+		
+
 }
 #include "webseed.moc"
