@@ -30,39 +30,41 @@
 #include <bcodec/bdecoder.h>
 #include <bcodec/bnode.h>
 #include <torrent/server.h>
-
+#include <torrent/torrent.h>
 #include "packetreader.h"
 #include "packetwriter.h"
 #include "peerdownloader.h"
 #include "peeruploader.h"
 #include "utpex.h"
+#include "peermanager.h"
+#include <net/reverseresolver.h>
 
 using namespace net;
 
 namespace bt
 {
 
-	
-	
 	static Uint32 peer_id_counter = 1;
+	bool Peer::resolve_hostname = true;
 	
 	
 	Peer::Peer(mse::StreamSocket* sock,const PeerID & peer_id,
-			   Uint32 num_chunks,Uint32 chunk_size,Uint32 support,bool local)
-	: sock(sock),pieces(num_chunks),peer_id(peer_id)
+			   Uint32 num_chunks,Uint32 chunk_size,Uint32 support,bool local,PeerManager* pman)
+	: sock(sock),pieces(num_chunks),peer_id(peer_id),pman(pman)
 	{
 		id = peer_id_counter;
 		peer_id_counter++;
 		
 		ut_pex = 0;
 		preader = new PacketReader(this);
-		choked = am_choked = true;
-		interested = am_interested = false;
+		stats.choked = true;
+		stats.interested = stats.am_interested = false;
 		killed = false;
 		downloader = new PeerDownloader(this,chunk_size);
 		uploader = new PeerUploader(this);
 		
 		
+		stalled_timer.update();
 		pwriter = new PacketWriter(this);
 		time_choked = GetCurrentTime();
 		time_unchoked = 0;
@@ -72,6 +74,8 @@ namespace bt
 		stats.client = peer_id.identifyClient();
 		stats.ip_address = getIPAddresss();
 		stats.choked = true;
+		stats.interested = false;
+		stats.am_interested = false;
 		stats.download_rate = 0;
 		stats.upload_rate = 0;
 		stats.perc_of_file = 0;
@@ -81,7 +85,6 @@ namespace bt
 		stats.extension_protocol = support & EXT_PROT_SUPPORT;
 		stats.bytes_downloaded = stats.bytes_uploaded = 0;
 		stats.aca_score = 0.0;
-		stats.evil = false;
 		stats.has_upload_slot = false;
 		stats.num_up_requests = stats.num_down_requests = 0;
 		stats.encrypted = sock->encrypted();
@@ -97,6 +100,13 @@ namespace bt
 		}
 		pex_allowed = stats.extension_protocol;
 		utorrent_pex_id = 0;
+		
+		if (resolve_hostname)
+		{
+			net::ReverseResolver* res = new net::ReverseResolver();
+			connect(res,SIGNAL(resolved(QString)),this,SLOT(resolved(QString)),Qt::QueuedConnection);
+			res->resolveAsync(sock->getRemoteAddress());
+		}
 	}
 
 
@@ -114,16 +124,12 @@ namespace bt
 	{
 		sock->close();
 	}
-	
 
 	void Peer::kill()
 	{
 		sock->close();
 		killed = true;
 	}
-
-	
-	
 	
 	void Peer::packetReady(const Uint8* packet,Uint32 len)
 	{
@@ -144,11 +150,11 @@ namespace bt
 					return;
 				}
 				
-				if (!choked)
+				if (!stats.choked)
 				{
 					time_choked = GetCurrentTime();
 				}
-				choked = true;
+				stats.choked = true;
 				downloader->choked();
 				break;
 			case UNCHOKE:
@@ -159,9 +165,9 @@ namespace bt
 					return;
 				}
 				
-				if (choked)
+				if (stats.choked)
 					time_unchoked = GetCurrentTime();
-				choked = false;
+				stats.choked = false;
 				break;
 			case INTERESTED:
 				if (len != 1)
@@ -170,10 +176,10 @@ namespace bt
 					kill();
 					return;
 				}
-				if (!interested)
+				if (!stats.interested)
 				{
-					interested = true;
-					rerunChoker();
+					stats.interested = true;
+					pman->rerunChoker();
 				}
 				break;
 			case NOT_INTERESTED:
@@ -183,10 +189,10 @@ namespace bt
 					kill();
 					return;
 				}
-				if (interested)
+				if (stats.interested)
 				{
-					interested = false;
-					rerunChoker();
+					stats.interested = false;
+					pman->rerunChoker();
 				}
 				break;
 			case HAVE:
@@ -200,7 +206,7 @@ namespace bt
 					Uint32 ch = ReadUint32(tmp_buf,1);
 					if (ch < pieces.getNumBits())
 					{
-						haveChunk(this,ch);
+						pman->have(this,ch);
 						pieces.set(ch,true);
 					}
 					else
@@ -219,7 +225,7 @@ namespace bt
 				}
 				
 				pieces = BitSet(tmp_buf+1,pieces.getNumBits());
-				bitSetReceived(pieces);
+				pman->bitSetReceived(this,pieces);
 				break;
 			case REQUEST:
 				if (len != 13)
@@ -236,7 +242,7 @@ namespace bt
 							ReadUint32(tmp_buf,9),
 							downloader);
 					
-					if (!am_choked)
+					if (stats.has_upload_slot)
 						uploader->addRequest(r);
 					else if (stats.fast_extensions)
 						pwriter->sendReject(r);
@@ -255,13 +261,12 @@ namespace bt
 				
 				{
 					stats.bytes_downloaded += (len - 9);
-					// turn on evil bit
-					if (stats.evil)
-						stats.evil = false;
 					Piece p(ReadUint32(tmp_buf,1),
 							ReadUint32(tmp_buf,5),
 							len - 9,downloader,tmp_buf+9);
-					piece(p);
+					downloader->piece(p);
+					pman->pieceReceived(p);
+					downloader->update();
 				}
 				break;
 			case CANCEL:
@@ -307,7 +312,7 @@ namespace bt
 				{
 					Uint16 port = ReadUint16(tmp_buf,1);
 				//	Out(SYS_CON|LOG_DEBUG) << "Got PORT packet : " << port << endl;
-					gotPortPacket(getIPAddresss(),port);
+					pman->portPacketReceived(getIPAddresss(),port);
 				}
 				break;
 			case HAVE_ALL:
@@ -318,7 +323,7 @@ namespace bt
 					return;
 				}
 				pieces.setAll(true);
-				bitSetReceived(pieces);
+				pman->bitSetReceived(this,pieces);
 				break;
 			case HAVE_NONE:
 				if (len != 1)
@@ -328,7 +333,7 @@ namespace bt
 					return;
 				}
 				pieces.setAll(false);
-				bitSetReceived(pieces);
+				pman->bitSetReceived(this,pieces);
 				break;
 			case SUGGEST_PIECE:
 				// ignore suggestions for the moment
@@ -367,7 +372,7 @@ namespace bt
 				// handshake packet, so just check if the peer supports ut_pex
 				dict = dict->getDict(QString("m"));
 				BValueNode* val = 0;
-				if (dict && (val = dict->getValue("ut_pex")))
+				if (dict && (val = dict->getValue("ut_pex")) && UTPex::isEnabled())
 				{
 					utorrent_pex_id = val->data().toInt();
 					if (ut_pex)
@@ -424,12 +429,6 @@ namespace bt
 	{
 		return sock->bytesAvailable();
 	}
-
-	void Peer::dataWritten(int )
-	{
-	//	Out(SYS_CON|LOG_DEBUG) << "dataWritten " << bytes << endl;
-		
-	}
 	
 	Uint32 Peer::getUploadRate() const 
 	{
@@ -447,12 +446,7 @@ namespace bt
 			return 0;
 	}
 	
-	bool Peer::readyToSend() const 
-	{
-		return true;
-	}
-	
-	void Peer::update(PeerManager* pman)
+	void Peer::update()
 	{
 		if (killed)
 			return;
@@ -477,6 +471,17 @@ namespace bt
 		
 		if (ut_pex && ut_pex->needsUpdate())
 			ut_pex->update(pman);
+
+		// if no data is being sent or recieved, and there are pending requests
+		// increment the connection stalled timer
+		if (getUploadRate() > 100 || getDownloadRate() > 100 || 
+		   (uploader->getNumRequests() == 0 && downloader->getNumRequests() == 0) )
+			stalled_timer.update();
+	}
+	
+	bool Peer::isStalled() const
+	{
+		return stalled_timer.getElapsedSinceUpdate() >= 2*60*1000;
 	}
 	
 	bool Peer::isSnubbed() const
@@ -521,12 +526,21 @@ namespace bt
 
 	float Peer::percentAvailable() const
 	{
-		return (float)pieces.numOnBits() / (float)pieces.getNumBits() * 100.0;
+		// calculation needs to use bytes, instead of chunks, because
+		// the last chunk can have a different size
+		const Torrent & tor = pman->getTorrent();
+		Uint64 bytes = 0; 
+		if (pieces.get(tor.getNumChunks() - 1))
+			bytes = tor.getChunkSize() * (pieces.numOnBits() - 1) + tor.getLastChunkSize();
+		else
+			bytes = tor.getChunkSize() * pieces.numOnBits();
+		
+		Uint64 tbytes = tor.getChunkSize() * (pieces.getNumBits() - 1) + tor.getLastChunkSize();
+		return (float)bytes / (float)tbytes * 100.0;
 	}
 
 	const PeerInterface::Stats & Peer::getStats() const
 	{
-		stats.choked = this->isChoked();
 		stats.download_rate = this->getDownloadRate();
 		stats.upload_rate = this->getUploadRate();
 		stats.perc_of_file = this->percentAvailable();
@@ -543,7 +557,7 @@ namespace bt
 	
 	void Peer::choke()
 	{
-		if (am_choked)
+		if (!stats.has_upload_slot)
 			return;
 		
 		pwriter->sendChoke();
@@ -552,12 +566,12 @@ namespace bt
 	
 	void Peer::emitPortPacket()
 	{
-		gotPortPacket(sock->getRemoteIPAddress(),sock->getRemotePort());
+		pman->portPacketReceived(sock->getRemoteIPAddress(),sock->getRemotePort());
 	}
 	
 	void Peer::emitPex(const QByteArray & data)
 	{
-		pex(data);
+		pman->pex(data);
 	}
 	
 	void Peer::setPexEnabled(bool on)
@@ -568,13 +582,13 @@ namespace bt
 		// send extension protocol handshake
 		bt::Uint16 port = Globals::instance().getServer().getPortInUse();
 		
-		if (ut_pex && !on)
+		if (ut_pex && (!on || !UTPex::isEnabled()))
 		{
 			delete ut_pex;
 			ut_pex = 0;
 			
 		}
-		else if (!ut_pex && on && utorrent_pex_id > 0)
+		else if (!ut_pex && on && utorrent_pex_id > 0 && UTPex::isEnabled())
 		{
 			// if the other side has enabled it to, create a new UTPex object
 			ut_pex = new UTPex(this,utorrent_pex_id);
@@ -589,6 +603,18 @@ namespace bt
 	{
 		sock->setGroupIDs(up_gid,down_gid);
 	}
+		
+	void Peer::resolved(const QString & hinfo)
+	{
+		stats.hostname = hinfo;
+	}
+	
+	void Peer::setResolveHostnames(bool on)
+	{
+		resolve_hostname = on;
+	}
+
+
 }
 
 #include "peer.moc"
