@@ -35,6 +35,8 @@
 #include <interfaces/trackerslist.h>
 #include <interfaces/monitorinterface.h>
 #include <interfaces/cachefactory.h>
+#include <interfaces/queuemanagerinterface.h>
+#include <interfaces/chunkselectorinterface.h>
 #include <datachecker/singledatachecker.h>
 #include <datachecker/multidatachecker.h>
 #include <datachecker/datacheckerlistener.h>
@@ -42,36 +44,33 @@
 #include <migrate/ccmigrate.h>
 #include <migrate/cachemigrate.h>
 #include <dht/dhtbase.h>
-
 #include <download/downloader.h>
 #include <download/webseed.h>
-#include "uploader.h"
-#include "peersourcemanager.h"
 #include <diskio/cache.h>
 #include <diskio/chunkmanager.h>
-#include "torrent.h"
-#include <peer/peermanager.h>
-
-#include "torrentfile.h"
-
+#include <diskio/preallocationthread.h>
+#include <diskio/preallocationjob.h>
+#include <diskio/movedatafilesjob.h>
+#include <datachecker/datacheckerjob.h>
 #include <peer/peer.h>
+#include <peer/peermanager.h>
+#include <peer/packetwriter.h>
+#include <net/socketmonitor.h>
+#include "torrentfile.h"
+#include "torrent.h"
 #include "choker.h"
-
 #include "globals.h"
 #include "server.h"
-#include <peer/packetwriter.h>
-#include <interfaces/queuemanagerinterface.h>
-#include <interfaces/chunkselectorinterface.h>
+#include "uploader.h"
+#include "peersourcemanager.h"
 #include "statsfile.h"
-#include <diskio/preallocationthread.h>
 #include "timeestimator.h"
-#include <net/socketmonitor.h>
+#include "jobqueue.h"
+
+
 
 namespace bt
 {
-
-	
-
 	KUrl TorrentControl::completed_dir;
 	bool TorrentControl::completed_datacheck = false;
 	Uint32 TorrentControl::min_diskspace = 100;
@@ -81,55 +80,29 @@ namespace bt
 	TorrentControl::TorrentControl()
 	: tor(0),psman(0),cman(0),pman(0),downloader(0),uploader(0),choke(0),tmon(0),prealloc(false)
 	{
+		job_queue = new JobQueue(this);
 		custom_selector_factory = 0;
 		cache_factory = 0;
-		istats.last_announce = 0;
-		stats.imported_bytes = 0;
-		stats.trk_bytes_downloaded = 0;
-		stats.trk_bytes_uploaded = 0;
-		stats.running = false;
-		stats.started = false;
-		stats.stopped_by_error = false;
-		stats.session_bytes_downloaded = 0;
-		stats.session_bytes_uploaded = 0;
 		istats.session_bytes_uploaded = 0;
 		old_tordir = QString();
-		stats.status = NOT_STARTED;
-		stats.autostart = false;
-		stats.user_controlled = false;
-		stats.priv_torrent = false;
-		stats.seeders_connected_to = stats.seeders_total = 0;
-		stats.leechers_connected_to = stats.leechers_total = 0;
-		stats.total_times_downloaded = 0;
-		stats.max_share_ratio = 0.00f;
-		stats.max_seed_time = 0;
-		stats.last_download_activity_time = stats.last_upload_activity_time = 0;
-		stats.tracker_status = TRACKER_OK;
+		
 		istats.running_time_dl = istats.running_time_ul = 0;
 		istats.prev_bytes_dl = 0;
 		istats.prev_bytes_ul = 0;
-		istats.trk_prev_bytes_dl = istats.trk_prev_bytes_ul = 0;
 		istats.io_error = false;
 		istats.priority = 0;
 		istats.custom_output_name = false;
 		istats.diskspace_warning_emitted = false;
-		updateStats();
-		prealloc_thread = 0;
-		dcheck_thread = 0;
 		istats.dht_on = false;
-		stats.num_corrupted_chunks = 0;
+		updateStats();
 		
 		m_eta = new TimeEstimator(this);
 		// by default no torrent limits
 		upload_gid = download_gid = 0;
 		upload_limit = download_limit = 0;
 		assured_upload_speed = assured_download_speed = 0;
-		moving_files = false;
 	}
 
-
-
-
 	TorrentControl::~TorrentControl()
 	{
 		if (stats.running)
@@ -141,6 +114,10 @@ namespace bt
 		
 		if (tmon)
 			tmon->destroyed();
+		
+		if (downloader)
+			downloader->saveWebSeeds(tordir + "webseeds");
+		
 		delete choke;
 		delete downloader;
 		delete uploader;
@@ -152,22 +129,17 @@ namespace bt
 		delete custom_selector_factory;
 		delete cache_factory;
 	}
-	
-	bool TorrentControl::updateNeeded() const
-	{
-		return stats.running;
-	}
 
 	void TorrentControl::update()
 	{
 		UpdateCurrentTime();
-		if (moving_files || dcheck_thread || prealloc_thread)
+		if (job_queue->runningJobs())
 			return;
 		
 		if (istats.io_error)
 		{
 			stop(false);
-			emit stoppedByError(this, error_msg);
+			emit stoppedByError(this, stats.error_msg);
 			return;
 		}
 		
@@ -209,7 +181,7 @@ namespace bt
 				finished(this);
 
 				//Move completed download to specified directory if needed
-				if (!completed_dir.path().isNull())
+				if (!completed_dir.toLocalFile().isNull())
 					moveCompleted = true;
 				
 				// See if we need to do a data check
@@ -225,11 +197,9 @@ namespace bt
 					psman->start();
 				else
 					psman->manualUpdate();
-				istats.last_announce = bt::GetCurrentTime();
 				istats.time_started_dl = QDateTime::currentDateTime();
 				// Tell QM to redo queue
-				if (!isUserControlled())
-					updateQueue();
+				updateQueue();
 			}
 			updateStatus();
 
@@ -292,19 +262,14 @@ namespace bt
 				stalled_timer.update();
 			}
 			
-			if (overMaxRatio() || overMaxSeedTime()) 
-			{ 
-				if (!stats.user_controlled) //if it's queued make sure to dequeue it 
-				{
-					setUserControlled(true);
-				}
-                 
-				stop(true); 
+			if (stats.completed && (overMaxRatio() || overMaxSeedTime()))
+			{
+				stop(); 
 				emit seedingAutoStopped(this, overMaxRatio() ? MAX_RATIO_REACHED : MAX_SEED_TIME_REACHED);
 			}
 
 			//Update diskspace if needed (every 1 min)			
-			if(!stats.completed && stats.running && bt::GetCurrentTime() - last_diskspace_check >= 60 * 1000)
+			if (!stats.completed && stats.running && bt::GetCurrentTime() - last_diskspace_check >= 60 * 1000)
 			{
 				checkDiskSpace(true);
 			}
@@ -313,19 +278,9 @@ namespace bt
 			if (checkOnCompletion || (auto_recheck && stats.num_corrupted_chunks >= num_corrupted_for_recheck))
 				needDataCheck(this);
 			
-			//Move completed files if needed:
+			// Move completed files if needed:
 			if (moveCompleted)
-			{
-				if (stats.status == CHECKING_DATA)
-				{
-					// wait for dcheck_thread to finish before moving the files
-					connect(this,SIGNAL(dataCheckFinished()),this,SLOT(moveToCompletedDir()));
-				}
-				else
-				{
-					moveToCompletedDir();
-				}
-			}
+				moveToCompletedDir();
 		}
 		catch (Error & e)
 		{
@@ -338,15 +293,15 @@ namespace bt
 		Out(SYS_DIO|LOG_IMPORTANT) << "Error : " << msg << endl;
 		stats.stopped_by_error = true;
 		stats.status = ERROR;
-		error_msg = msg;
+		stats.error_msg = msg;
 		istats.io_error = true;
 		statusChanged(this);
 	}
 
 	void TorrentControl::start()
 	{	
-		// do not start running torrents
-		if (stats.running || stats.status == ALLOCATING_DISKSPACE || moving_files)
+		// do not start running torrents or when there is a job running
+		if (stats.running || job_queue->runningJobs())
 			return;
 
 		stats.stopped_by_error = false;
@@ -377,7 +332,6 @@ namespace bt
 		}
 		
 		istats.time_started_ul = istats.time_started_dl = QDateTime::currentDateTime();
-		resetTrackerStats();
 		
 		if (prealloc)
 		{
@@ -385,12 +339,9 @@ namespace bt
 			if (Cache::preallocationEnabled() && !cman->haveAllChunks())
 			{
 				Out(SYS_GEN|LOG_NOTICE) << "Pre-allocating diskspace" << endl;
-				prealloc_thread = new PreallocationThread(cman);
-				connect(prealloc_thread,SIGNAL(finished()),this,SLOT(preallocThreadDone()),Qt::QueuedConnection);
 				stats.running = true;
-				stats.status = ALLOCATING_DISKSPACE;
-				prealloc_thread->start();
-				statusChanged(this);
+				job_queue->enqueue(new PreallocationJob(cman,this));
+				updateStatus();
 				return;
 			}
 			else
@@ -421,7 +372,7 @@ namespace bt
 		loadStats();
 		stats.running = true;
 		stats.started = true;
-		stats.autostart = true;
+		stats.queued = false;
 		stats.last_download_activity_time = stats.last_upload_activity_time = GetCurrentTime();
 		choker_update_timer.update();
 		stats_save_timer.update();
@@ -429,12 +380,11 @@ namespace bt
 		
 		stalled_timer.update();
 		psman->start();
-		istats.last_announce = bt::GetCurrentTime();
 		stalled_timer.update();
 	}
 		
 
-	void TorrentControl::stop(bool user,WaitJob* wjob)
+	void TorrentControl::stop(WaitJob* wjob)
 	{
 		QDateTime now = QDateTime::currentDateTime();
 		if(!stats.completed)
@@ -442,17 +392,9 @@ namespace bt
 		istats.running_time_ul += istats.time_started_ul.secsTo(now);
 		istats.time_started_ul = istats.time_started_dl = now;
 		
-		// stop preallocation thread if necesarry
-		if (prealloc_thread)
-		{
-			disconnect(prealloc_thread,SIGNAL(finished()),this,SLOT(preallocThreadDone()));
-			prealloc_thread->stop();
-			prealloc_thread->wait();
-			if (prealloc_thread->errorHappened() || prealloc_thread->isNotFinished())
-				saveStats(); // save stats, so that we will start preallocating the next time
-			prealloc_thread->deleteLater();
-			prealloc_thread = 0;
-		}
+		// stop preallocation
+		if (job_queue->currentJob() && job_queue->currentJob()->torrentStatus() == ALLOCATING_DISKSPACE)
+			job_queue->currentJob()->kill(false);
 	
 		if (stats.running)
 		{
@@ -474,13 +416,7 @@ namespace bt
 			
 			downloader->clearDownloads();
 		}
-		
-		if (user)
-		{
-			//make this torrent user controlled
-			setUserControlled(true);
-			stats.autostart = false;
-		}
+	
 		
 		pman->savePeerList(tordir + "peer_list");
 		pman->stop();
@@ -489,11 +425,11 @@ namespace bt
 		cman->stop();
 		
 		stats.running = false;
+		stats.autostart = wjob != 0;
+		stats.queued = false;
 		saveStats();
 		updateStatus();
 		updateStats();
-		stats.trk_bytes_downloaded = 0;
-		stats.trk_bytes_uploaded = 0;
 
 		emit torrentStopped(this);
 	}
@@ -516,8 +452,7 @@ namespace bt
 	void TorrentControl::init(QueueManagerInterface* qman,
 							  const QString & torrent,
 							  const QString & tmpdir,
-							  const QString & ddir,
-							  const QString & default_save_dir)
+							  const QString & ddir)
 	{
 		// first load the torrent file
 		tor = new Torrent();
@@ -535,7 +470,7 @@ namespace bt
 					"The torrent is probably corrupt or is not a valid torrent file.",torrent,err.toString()));
 		}
 		
-		initInternal(qman,tmpdir,ddir,default_save_dir,torrent.startsWith(tmpdir));
+		initInternal(qman,tmpdir,ddir);
 		
 		// copy torrent in tor dir
 		QString tor_copy = tordir + "torrent";
@@ -546,8 +481,7 @@ namespace bt
 	}
 	
 	
-	void TorrentControl::init(QueueManagerInterface* qman, const QByteArray & data,const QString & tmpdir,
-							  const QString & ddir,const QString & default_save_dir)
+	void TorrentControl::init(QueueManagerInterface* qman, const QByteArray & data,const QString & tmpdir,const QString & ddir)
 	{
 		// first load the torrent file
 		tor = new Torrent();
@@ -565,7 +499,7 @@ namespace bt
 				"The torrent is probably corrupt or is not a valid torrent file.",err.toString()));
 		}
 		
-		initInternal(qman,tmpdir,ddir,default_save_dir,true);
+		initInternal(qman,tmpdir,ddir);
 		// copy data into torrent file
 		QString tor_copy = tordir + "torrent";
 		QFile fptr(tor_copy);
@@ -617,7 +551,7 @@ namespace bt
 		stats.running = false;
 		stats.torrent_name = tor->getNameSuggestion();
 		stats.multi_file_torrent = tor->isMultiFile();
-		stats.total_bytes = tor->getFileLength();
+		stats.total_bytes = tor->getTotalSize();
 		stats.priv_torrent = tor->isPrivate();
 		
 		// check the stats file for the custom_output_name variable
@@ -638,8 +572,6 @@ namespace bt
 		pman = new PeerManager(*tor);
 		//Out() << "Tracker url " << url << " " << url.protocol() << " " << url.prettyURL() << endl;
 		psman = new PeerSourceManager(this,pman);
-		connect(psman,SIGNAL(statusChanged(TrackerStatus , const QString& )),
-				this,SLOT(trackerStatusChanged(TrackerStatus , const QString& )));
 
 		// Create chunkmanager, load the index file if it exists
 		// else create all the necesarry files
@@ -666,31 +598,12 @@ namespace bt
 		connect(cman,SIGNAL(corrupted( Uint32 )),this,SLOT(corrupted( Uint32 )));
 	}
 	
-	void TorrentControl::initInternal(QueueManagerInterface* qman,
-									  const QString & tmpdir,
-									  const QString & ddir,
-									  const QString & default_save_dir,
-									  bool first_time)
+	void TorrentControl::initInternal(QueueManagerInterface* qman,const QString & tmpdir,const QString & ddir)
 	{
 		checkExisting(qman);
 		setupDirs(tmpdir,ddir);
 		setupStats();
 		loadEncoding();
-
-		if (!first_time)
-		{
-			// if we do not need to copy the torrent, it is an existing download and we need to see
-			// if it is not an old download
-			try
-			{
-				migrateTorrent(default_save_dir);
-			}
-			catch (Error & err)
-			{
-				
-				throw Error(i18n("Cannot migrate %1 : %2",tor->getNameSuggestion(),err.toString()));
-			}
-		}
 		setupData();
 		updateStatus();
 		
@@ -727,21 +640,14 @@ namespace bt
 
 	bool TorrentControl::announceAllowed()
 	{
-		if(istats.last_announce == 0)
-			return true;
-		
-		if (psman && psman->getNumFailures() == 0)
-			return bt::GetCurrentTime() - istats.last_announce >= 60 * 1000;
-		else
-			return true;
+		return psman != 0 && stats.running;
 	}
 	
 	void TorrentControl::updateTracker()
 	{
-		if (stats.running && announceAllowed())
+		if (announceAllowed())
 		{
 			psman->manualUpdate();
-			istats.last_announce = bt::GetCurrentTime();
 		}
 	}
 	
@@ -752,9 +658,6 @@ namespace bt
 
 	void TorrentControl::onNewPeer(Peer* p)
 	{
-		connect(p,SIGNAL(gotPortPacket( const QString&, Uint16 )),
-				this,SLOT(onPortPacket( const QString&, Uint16 )));
-		
 		if (p->getStats().fast_extensions)
 		{
 			const BitSet & bs = cman->getBitSet();
@@ -795,8 +698,6 @@ namespace bt
 
 	void TorrentControl::onPeerRemoved(Peer* p)
 	{
-		disconnect(p,SIGNAL(gotPortPacket( const QString&, Uint16 )),
-				this,SLOT(onPortPacket( const QString&, Uint16 )));
 		if (tmon)
 			tmon->peerRemoved(p);
 	}
@@ -837,18 +738,10 @@ namespace bt
 	bool TorrentControl::changeOutputDir(const QString & ndir,int flags)
 	{
 		//check if torrent is running and stop it before moving data
-		restart_torrent_after_move_data_files = false;
-		if(stats.running)
-		{
-			restart_torrent_after_move_data_files = true;
-			this->stop(false);
-		}
-		
 		QString new_dir = ndir;
 		if (!new_dir.endsWith(bt::DirSeparator()))
 			new_dir += bt::DirSeparator();
 		
-		moving_files = true;
 		try
 		{
 			QString nd;
@@ -872,7 +765,7 @@ namespace bt
 			if (stats.output_path != nd)
 			{
 				move_data_files_destination_path = nd;
-				KJob* j = 0;
+				Job* j = 0;
 				if (flags & bt::TorrentInterface::MOVE_FILES)
 				{
 					if (stats.multi_file_torrent)
@@ -884,6 +777,7 @@ namespace bt
 				if (j)
 				{
 					connect(j,SIGNAL(result(KJob*)),this,SLOT(moveDataFilesFinished(KJob*)));
+					job_queue->enqueue(j);
 					return true;
 				}
 				else
@@ -899,19 +793,15 @@ namespace bt
 		catch (Error& err)
 		{			
 			Out(SYS_GEN|LOG_IMPORTANT) << "Could not move " << stats.output_path << " to " << new_dir << ". Exception: " << endl;
-			moving_files = false;
 			return false;
 		}
 		
-		moving_files = false;
-		if(restart_torrent_after_move_data_files)
-			this->start();
-		
 		return true;
 	}
 	
-	void TorrentControl::moveDataFilesFinished(KJob* job)
+	void TorrentControl::moveDataFilesFinished(KJob* kj)
 	{
+		Job* job = (Job*)kj;
 		if (job)
 			cman->moveDataFilesFinished(job);
 		
@@ -928,45 +818,39 @@ namespace bt
 		{
 			Out(SYS_GEN|LOG_IMPORTANT) << "Could not move " << stats.output_path << " to " << move_data_files_destination_path << endl;
 		}
-
-		moving_files = false;
-		if (restart_torrent_after_move_data_files)
-		{
-			this->start();
-		}
 	}
 
 	bool TorrentControl::moveTorrentFiles(const QMap<TorrentFileInterface*,QString> & files)
 	{
-		bool start = false;
-		
-		// check if torrent is running and stop it before moving data
-		if(stats.running)
-		{
-			start = true;
-			this->stop(false);
-		}
-	
-		moving_files = true;
 		try
 		{
-			KJob* j = cman->moveDataFiles(files);
-			if (j && j->exec())
-				cman->moveDataFilesFinished(files,j);
-			Out(SYS_GEN|LOG_NOTICE) << "Move of data files completed " << endl;
+			Job* j = cman->moveDataFiles(files);
+			if (j)
+			{
+				connect(j,SIGNAL(result(KJob*)),this,SLOT(moveDataFilesWithMapFinished(KJob*)));
+				job_queue->enqueue(j);
+			}
+			
 		}
 		catch (Error& err)
-		{			
-			moving_files = false;
+		{
 			return false;
 		}
 	
-		moving_files = false;
-		if(start)
-			this->start();
-		
 		return true;
 	}
+	
+	
+	void TorrentControl::moveDataFilesWithMapFinished(KJob* j)
+	{
+		if (!j)
+			return;
+		
+		MoveDataFilesJob* job = (MoveDataFilesJob*)j;
+		cman->moveDataFilesFinished(job->fileMap(),job);
+		Out(SYS_GEN|LOG_NOTICE) << "Move of data files completed " << endl;
+	}
+
 
 	void TorrentControl::rollback()
 	{
@@ -987,24 +871,23 @@ namespace bt
 		TorrentStatus old = stats.status;
 		if (stats.stopped_by_error)
 			stats.status = ERROR;
-		else if (dcheck_thread)
-			stats.status = CHECKING_DATA;
-		else if (!stats.started && stats.user_controlled)
-			stats.status = NOT_STARTED;
-		else if (!stats.running && !stats.user_controlled)
+		else if (job_queue->currentJob() && job_queue->currentJob()->torrentStatus() != INVALID_STATUS)
+			stats.status = job_queue->currentJob()->torrentStatus();
+		else if (stats.queued)
 			stats.status = QUEUED;
-		else if (!stats.running && stats.completed && (overMaxRatio() || overMaxSeedTime()))
+		else if (stats.completed && (overMaxRatio() || overMaxSeedTime()))
 			stats.status = SEEDING_COMPLETE;
 		else if (!stats.running && stats.completed)
 			stats.status = DOWNLOAD_COMPLETE;
+		else if (!stats.started)
+			stats.status = NOT_STARTED;
 		else if (!stats.running)
 			stats.status = STOPPED;
 		else if (stats.running && stats.completed)
 			stats.status = SEEDING;
 		else if (stats.running) 
 			// protocol messages are also included in speed calculation, so lets not compare with 0
-			stats.status = downloader->downloadRate() > 100 ?
-					DOWNLOADING : STALLED;
+			stats.status = downloader->downloadRate() > 100 ? DOWNLOADING : STALLED;
 		
 		if (old != stats.status)
 			statusChanged(this);
@@ -1065,8 +948,8 @@ namespace bt
 			st.write("RUNNING_TIME_UL", QString("%1").arg(istats.running_time_ul));
 		}
 		
+		st.write("QM_CAN_START",stats.qm_can_start ? "1" : "0");
 		st.write("PRIORITY", QString("%1").arg(istats.priority));
-		st.write("USER_CONTROLLED",stats.user_controlled ? "1" : "0");
 		st.write("AUTOSTART", QString("%1").arg(stats.autostart));
 		st.write("IMPORTED", QString("%1").arg(stats.imported_bytes));
 		st.write("CUSTOM_OUTPUT_NAME",istats.custom_output_name ? "1" : "0");
@@ -1131,17 +1014,11 @@ namespace bt
 			display_name = st.readString("DISPLAY_NAME");
 		
 		setPriority(st.readInt("PRIORITY"));
-		// before USER_CONTROLLED was added, priority == 0 meant user controlled
-		if (st.hasKey("USER_CONTROLLED"))
-			setUserControlled(st.readBoolean("USER_CONTROLLED"));
-		else
-			setUserControlled(getPriority() == 0); 
 		stats.autostart = st.readBoolean("AUTOSTART");
-		
 		stats.imported_bytes = st.readUint64("IMPORTED");
 		stats.max_share_ratio = st.readFloat("MAX_RATIO");
 		stats.max_seed_time = st.readFloat("MAX_SEED_TIME");
-
+		stats.qm_can_start = st.readBoolean("QM_CAN_START");
 
 		if (st.hasKey("RESTART_DISK_PREALLOCATION"))
 			prealloc = st.readString("RESTART_DISK_PREALLOCATION") == "1";
@@ -1227,14 +1104,6 @@ namespace bt
 		return !tor->isMultiFile() && tor->isMultimedia();
 	}
 
-	Uint32 TorrentControl::getTimeToNextTrackerUpdate() const
-	{
-		if (psman)
-			return psman->getTimeToNextUpdate();
-		else
-			return 0;
-	}
-
 	void TorrentControl::updateStats()
 	{
 		stats.num_chunks_downloading = downloader ? downloader->numActiveDownloads() : 0;
@@ -1250,7 +1119,7 @@ namespace bt
 		stats.num_chunks_excluded = cman ? cman->chunksExcluded() : 0;
 		stats.chunk_size = tor ? tor->getChunkSize() : 0;
 		stats.num_chunks_left = cman ? cman->chunksLeft() : 0;
-		stats.total_bytes_to_download = (tor && cman) ?	tor->getFileLength() - cman->bytesExcluded() : 0;
+		stats.total_bytes_to_download = (tor && cman) ? tor->getTotalSize() - cman->bytesExcluded() : 0;
 		
 		
 		
@@ -1263,24 +1132,10 @@ namespace bt
 			stats.session_bytes_uploaded = (stats.bytes_uploaded - istats.prev_bytes_ul) + istats.session_bytes_uploaded;
 		else
 			stats.session_bytes_uploaded = istats.session_bytes_uploaded;
-		/*
-			Safety check, it is possible that stats.bytes_downloaded gets subtracted in Downloader.
-			Which can cause stats.bytes_downloaded to be smaller the istats.trk_prev_bytes_dl.
-			This can screw up your download ratio.
-		*/
-		if (stats.bytes_downloaded >= istats.trk_prev_bytes_dl)
-			stats.trk_bytes_downloaded = stats.bytes_downloaded - istats.trk_prev_bytes_dl;
-		else
-			stats.trk_bytes_downloaded = 0;
-		
-		if (stats.bytes_uploaded >= istats.trk_prev_bytes_ul)
-			stats.trk_bytes_uploaded = stats.bytes_uploaded - istats.trk_prev_bytes_ul;
-		else
-			stats.trk_bytes_uploaded = 0;
+
 		
 		getSeederInfo(stats.seeders_total,stats.seeders_connected_to);
 		getLeecherInfo(stats.leechers_total,stats.leechers_connected_to);
-		stats.total_times_downloaded = psman ? psman->getTotalTimesDownloaded() : 0;
 	}
 
 	void TorrentControl::trackerScrapeDone()
@@ -1362,53 +1217,6 @@ namespace bt
 		else
 			return TorrentFile::null;
 	}
-
-	void TorrentControl::migrateTorrent(const QString & default_save_dir)
-	{
-		if (bt::Exists(tordir + "current_chunks") && bt::IsPreMMap(tordir + "current_chunks"))
-		{
-			// in case of error copy torX dir to migrate-failed-tor
-			QString dd = tordir;
-			int pos = dd.lastIndexOf("tor");
-			if (pos != - 1)
-			{
-				dd = dd.replace(pos,3,"migrate-failed-tor");
-				Out(SYS_GEN|LOG_DEBUG) << "Copying " << tordir << " to " << dd << endl;
-				bt::CopyDir(tordir,dd,true);
-			}
-				
-			bt::MigrateCurrentChunks(*tor,tordir + "current_chunks");
-			if (outputdir.isNull() && bt::IsCacheMigrateNeeded(*tor,tordir + "cache"))
-			{
-				// if the output dir is NULL
-				if (default_save_dir.isNull())
-				{
-					KMessageBox::information(0,
-						i18n("The torrent %1 was started with a previous version of KTorrent."
-							" To make sure this torrent still works with this version of KTorrent, "
-							"we will migrate this torrent. You will be asked for a location to save "
-							"the torrent to. If you press cancel, we will select your home directory.",
-							tor->getNameSuggestion()));
-					outputdir = KFileDialog::getExistingDirectory(KUrl("kfiledialog:///openTorrent"), 0,i18n("Select Folder to Save To"));
-					if (outputdir.isNull())
-						outputdir = QDir::homePath();
-				}
-				else
-				{
-					outputdir = default_save_dir;
-				}
-				
-				if (!outputdir.endsWith(bt::DirSeparator()))
-					outputdir += bt::DirSeparator();
-				
-				bt::MigrateCache(*tor,tordir + "cache",outputdir);
-			}
-			
-			// delete backup
-			if (pos != - 1)
-				bt::Delete(dd);
-		}
-	}
 	
 	void TorrentControl::setPriority(int p)
 	{
@@ -1417,26 +1225,16 @@ namespace bt
 		updateStatus();
 	}
 	
-	void TorrentControl::setUserControlled(bool uc)
-	{
-		stats.user_controlled = uc;
-		saveStats();
-		updateStatus();
-	}
-	
 	void TorrentControl::setMaxShareRatio(float ratio)
 	{
-		if(ratio == 1.00f)
+		if (ratio == 1.00f)
 		{
-			if(stats.max_share_ratio != ratio)
+			if (stats.max_share_ratio != ratio)
 				stats.max_share_ratio = ratio;
 		}
 		else
 			stats.max_share_ratio = ratio;
 		
-		if(stats.completed && !stats.running && !stats.user_controlled && (ShareRatio(stats) >= stats.max_share_ratio))
-			setUserControlled(true); //dequeue it
-		
 		saveStats();
 		emit maxRatioChanged(this);
 	}
@@ -1449,13 +1247,7 @@ namespace bt
 
 	bool TorrentControl::overMaxRatio()
 	{
-		if(stats.completed && stats.max_share_ratio > 0)
-		{
-			if(ShareRatio(stats) >= stats.max_share_ratio)
-				return true;
-		}
-		
-		return false;
+		return stats.overMaxRatio();
 	}
 
 	bool TorrentControl::overMaxSeedTime()
@@ -1470,38 +1262,6 @@ namespace bt
 		
 		return false;
 	}
-	
-	QString TorrentControl::statusToString() const
-	{
-		switch (stats.status)
-		{
-			case NOT_STARTED :
-				return i18n("Not started");
-			case DOWNLOAD_COMPLETE :
-				return i18n("Download completed");
-			case SEEDING_COMPLETE :
-				return i18n("Seeding completed");
-			case SEEDING :
-				return i18n("Seeding");
-			case DOWNLOADING:
-				return i18n("Downloading");
-			case STALLED:
-				return i18n("Stalled");
-			case STOPPED:
-				return i18n("Stopped");
-			case ERROR :
-				return i18n("Error: ") + getShortErrorMessage(); 
-			case ALLOCATING_DISKSPACE:
-				return i18n("Allocating diskspace");
-			case QUEUED:
-				return i18n("Queued");
-			case CHECKING_DATA:
-				return i18n("Checking data");
-			case NO_SPACE_LEFT:
-				return i18n("Stopped. No space left on device.");
-		}
-		return QString();
-	}
 
 	TrackersList* TorrentControl::getTrackersList()
 	{
@@ -1512,55 +1272,36 @@ namespace bt
 	{
 		return psman;
 	}
-
-	void TorrentControl::onPortPacket(const QString & ip,Uint16 port)
-	{
-		if (Globals::instance().getDHT().isRunning() && !stats.priv_torrent)
-			Globals::instance().getDHT().portReceived(ip,port);
-	}
 	
 	void TorrentControl::startDataCheck(bt::DataCheckerListener* lst)
 	{
-		if (stats.status == ALLOCATING_DISKSPACE)
-			return;
-		
-		DataChecker* dc = 0;
-		stats.status = CHECKING_DATA;
-		stats.num_corrupted_chunks = 0; // reset the number of corrupted chunks found
-		if (stats.multi_file_torrent)
-			dc = new MultiDataChecker();
-		else
-			dc = new SingleDataChecker();
-	
-		dc->setListener(lst);
-		
-		dcheck_thread = new DataCheckerThread(dc,cman->getBitSet(),stats.output_path,*tor,tordir + "dnd" + bt::DirSeparator());
-		connect(dcheck_thread,SIGNAL(finished()),this,SLOT(afterDataCheck()),Qt::QueuedConnection);
-		
-		// dc->check(stats.output_path,*tor,tordir + "dnd" + bt::DirSeparator());
-		dcheck_thread->start(QThread::IdlePriority);
-		statusChanged(this);
+		job_queue->enqueue(new DataCheckerJob(lst,this));
 	}
 	
-	void TorrentControl::afterDataCheck()
+	void TorrentControl::beforeDataCheck()
 	{
-		DataChecker* dc = dcheck_thread->getDataChecker();
-		DataCheckerListener* lst = dc->getListener();
-		
-		bool err = !dcheck_thread->getError().isNull();
+		stats.status = CHECKING_DATA;
+		stats.num_corrupted_chunks = 0; // reset the number of corrupted chunks found
+		statusChanged(this);
+	}
+
+	
+	void TorrentControl::afterDataCheck(DataCheckerListener* lst,const BitSet & result,const QString & error)
+	{
+		bool err = !error.isNull();
 		if (err)
 		{
 			// show a queued error message when an error has occurred
-			KMessageBox::queuedMessageBox(0,KMessageBox::Error,dcheck_thread->getError());
+			KMessageBox::queuedMessageBox(0,KMessageBox::Error,error);
 			lst->stop();
 		}
 		
 		bool completed = stats.completed;
 		if (lst && !lst->isStopped())
 		{
-			downloader->dataChecked(dc->getResult());
+			downloader->dataChecked(result);
 			// update chunk manager
-			cman->dataChecked(dc->getResult());
+			cman->dataChecked(result);
 			if (lst->isAutoImport())
 			{
 				downloader->recalcDownloaded();
@@ -1580,10 +1321,7 @@ namespace bt
 		}
 			
 		updateStats();
-		dcheck_thread->deleteLater();
-		dcheck_thread = 0;
 		Out(SYS_GEN|LOG_NOTICE) << "Data check finished" << endl;
-		resetTrackerStats();
 		updateStatus();
 		if (lst)
 			lst->finished();
@@ -1595,20 +1333,11 @@ namespace bt
 			// Tell QM to redo queue 
 			// seeder might have become downloader, so 
 			// queue might need to be redone
-			updateQueue();
+			// use QTimer because we need to ensure this is run after the JobQueue removes the job
+			QTimer::singleShot(0,this,SIGNAL(updateQueue()));
 		}
 	}
 	
-	bool TorrentControl::isCheckingData(bool & finished) const
-	{
-		if (dcheck_thread)
-		{
-			finished = !dcheck_thread->isRunning();
-			return true;
-		}
-		return false;
-	}
-	
 	void TorrentControl::markExistingFilesAsDownloaded()
 	{
 		cman->markExistingFilesAsDownloaded();
@@ -1678,7 +1407,9 @@ namespace bt
 
 	void TorrentControl::deleteDataFiles()
 	{
-		cman->deleteDataFiles();
+		Job* job = cman->deleteDataFiles();
+		if (job)
+			job->start(); // don't use queue, this is only done when removing the torrent
 	}
 	
 	const bt::SHA1Hash & TorrentControl::getInfoHash() const
@@ -1686,20 +1417,6 @@ namespace bt
 		return tor->getInfoHash();
 	}
 	
-	void TorrentControl::resetTrackerStats()
-	{
-		istats.trk_prev_bytes_dl = stats.bytes_downloaded,
-		istats.trk_prev_bytes_ul = stats.bytes_uploaded,
-		stats.trk_bytes_downloaded = 0;
-		stats.trk_bytes_uploaded = 0;
-	}
-	
-	void TorrentControl::trackerStatusChanged(TrackerStatus s,const QString & ns)
-	{
-		stats.tracker_status = s;
-		stats.tracker_status_string = ns;
-	}
-	
 	void TorrentControl::addPeerSource(PeerSource* ps)
 	{
 		if (psman)
@@ -1934,25 +1651,19 @@ namespace bt
 		return pman;
 	}
 	
-	void TorrentControl::preallocThreadDone()
+	void TorrentControl::preallocFinished(const QString & error,bool completed)
 	{
-		if (!prealloc_thread)
-			return;
-		
-		// thread done
-		if (prealloc_thread->errorHappened())
+		Out(SYS_GEN|LOG_DEBUG) << "preallocFinished "<< error << " " << completed << endl;
+		if (!error.isEmpty() || !completed)
 		{
 			// upon error just call onIOError and return
-			onIOError(prealloc_thread->errorMessage());
-			prealloc_thread->deleteLater();
-			prealloc_thread = 0;
+			if (!error.isEmpty())
+				onIOError(error);
 			prealloc = true; // still need to do preallocation
 		}
 		else
 		{
 			// continue the startup of the torrent
-			prealloc_thread->deleteLater();
-			prealloc_thread = 0;
 			prealloc = false;
 			stats.status = NOT_STARTED;
 			saveStats();
@@ -1998,6 +1709,11 @@ namespace bt
 		return downloader->getWebSeed(i);
 	}
 	
+	WebSeedInterface* TorrentControl::getWebSeed(Uint32 i)
+	{
+		return downloader->getWebSeed(i);
+	}
+	
 	bool TorrentControl::addWebSeed(const KUrl & url)
 	{
 		WebSeed* ws = downloader->addWebSeed(url);
@@ -2036,19 +1752,45 @@ namespace bt
 	
 	void TorrentControl::moveToCompletedDir()
 	{
-		disconnect(this,SIGNAL(dataCheckFinished()),this,SLOT(moveToCompletedDir()));
-		
-		// it is possible that the user might have disabled moving to the completed dir during the data check
-		// so double check before we start the move
-		if (completed_dir.path().isNull() || !stats.completed)
-			return;
-			
-		QString outdir = completed_dir.path();
+		QString outdir = completed_dir.toLocalFile();
 		if (!outdir.endsWith(bt::DirSeparator()))
 			outdir += bt::DirSeparator();
 					
 		changeOutputDir(outdir,bt::TorrentInterface::MOVE_FILES);
 	}
+	
+	void TorrentControl::setAllowedToStart(bool on)
+	{
+		stats.qm_can_start = on;
+		if (on && stats.stopped_by_error)
+		{
+			// clear the error so user can restart the torrent
+			stats.stopped_by_error = false;
+		}
+		saveStats();
+	}
+	
+	void TorrentControl::setQueued(bool queued) 
+	{
+		stats.queued = queued;
+		updateStatus();
+	}
+	
+	QString TorrentControl::getComments() const
+	{
+		return tor->getComments();
+	}
+	
+	
+	void TorrentControl::allJobsDone()
+	{
+		updateStatus();
+		// update the QM to be sure
+		updateQueue();
+		runningJobsDone(this);
+	}
+
+
 }
 
 #include "torrentcontrol.moc"
