@@ -37,7 +37,7 @@ TransferKio::TransferKio(TransferGroup * parent, TransferFactory * factory,
       m_verifier(0),
       m_signature(0)
 {
-
+    setCapabilities(Transfer::Cap_Moving | Transfer::Cap_Renaming | Transfer::Cap_Resuming);//TODO check if it really can resume
 }
 
 bool TransferKio::setDirectory(const KUrl& newDirectory)
@@ -49,7 +49,7 @@ bool TransferKio::setDirectory(const KUrl& newDirectory)
 
 bool TransferKio::setNewDestination(const KUrl &newDestination)
 {
-    if (isResumable() && newDestination.isValid() && (newDestination != dest())) {
+    if (newDestination.isValid() && (newDestination != dest())) {
         KUrl oldPath = KUrl(m_dest.path() + ".part");
         if (oldPath.isValid() && QFile::exists(oldPath.pathOrUrl())) {
             m_movingFile = true;
@@ -90,8 +90,7 @@ void TransferKio::newDestResult(KJob *result)
 
 void TransferKio::start()
 {
-    if (!m_movingFile)
-    {
+    if (!m_movingFile && (status() != Finished)) {
         m_stopped = false;
         if(!m_copyjob)
             createJob();
@@ -104,8 +103,9 @@ void TransferKio::start()
 
 void TransferKio::stop()
 {
-    if(status() == Stopped)
+    if ((status() == Stopped) || (status() == Finished)) {
         return;
+    }
 
     m_stopped = true;
 
@@ -121,11 +121,6 @@ void TransferKio::stop()
     setTransferChange(Tc_Status | Tc_DownloadSpeed, true);
 }
 
-bool TransferKio::isResumable() const
-{
-    return true;
-}
-
 void TransferKio::deinit()
 {
     if (status() != Job::Finished)//if the transfer is not finished, we delete the *.part-file
