@@ -50,7 +50,8 @@ void TransferMultiSegKio::init()
 
     if (!m_dataSourceFactory)
     {
-        m_dataSourceFactory = new DataSourceFactory(m_dest, 0, 500 * 1024, this);
+         m_dataSourceFactory = new DataSourceFactory(this, m_dest);
+        connect(m_dataSourceFactory, SIGNAL(capabilitiesChanged()), this, SLOT(slotUpdateCapabilities()));
         connect(m_dataSourceFactory, SIGNAL(speed(ulong)), this, SLOT(slotSpeed(ulong)));
         connect(m_dataSourceFactory, SIGNAL(percent(ulong)), this, SLOT(slotPercent(ulong)));
         connect(m_dataSourceFactory, SIGNAL(processedSize(KIO::filesize_t)), this, SLOT(slotProcessedSize(KIO::filesize_t)));
@@ -59,6 +60,8 @@ void TransferMultiSegKio::init()
         connect(m_dataSourceFactory->verifier(), SIGNAL(verified(bool)), this, SLOT(slotVerified(bool)));
 
         m_dataSourceFactory->addMirror(m_source, MultiSegKioSettings::segments());
+
+        slotUpdateCapabilities();
     }
 }
 
@@ -107,11 +110,6 @@ void TransferMultiSegKio::stop()
     }
 }
 
-bool TransferMultiSegKio::isResumable() const
-{
-    return true;
-}
-
 bool TransferMultiSegKio::repair(const KUrl &file)
 {
     if (!file.isValid() || (m_dest == file))
@@ -119,6 +117,7 @@ bool TransferMultiSegKio::repair(const KUrl &file)
         if (m_dataSourceFactory && (m_dataSourceFactory->verifier()->status() == Verifier::NotVerified))
         {
             m_dataSourceFactory->repair();
+            return true;
         }
     }
 
@@ -135,7 +134,7 @@ bool TransferMultiSegKio::setDirectory(const KUrl& newDirectory)
 bool TransferMultiSegKio::setNewDestination(const KUrl &newDestination)
 {
     kDebug(5001) << "New destination: " << newDestination;
-    if (/*isResumable() && */newDestination.isValid() && (newDestination != dest()) && m_dataSourceFactory)
+    if (newDestination.isValid() && (newDestination != dest()) && m_dataSourceFactory)
     {
         m_movingFile = true;
         stop();
@@ -184,6 +183,11 @@ void TransferMultiSegKio::slotStatus(Job::Status status)
         QModelIndex statusIndex = m_fileModel->index(m_dest, FileItem::Status);
         m_fileModel->setData(statusIndex, status);
     }
+
+    if (status == Job::Finished) {
+        //when finished the file can be renamed/moved in any case
+        setCapabilities( capabilities() | Transfer::Cap_Moving | Transfer::Cap_Renaming);
+    }
 }
 
 void TransferMultiSegKio::slotVerified(bool isVerified)
@@ -215,21 +219,9 @@ void TransferMultiSegKio::slotPercent(ulong percent)
 
 void TransferMultiSegKio::slotProcessedSize(KIO::filesize_t processedSize)
 {
-    m_downloadedSize = processedSize;
-
-    setTransferChange(Tc_DownloadedSize, true);
-}
-
-void TransferMultiSegKio::slotSpeed(unsigned long bytes_per_second)
-{
-    kDebug(5001) << "slotSpeed: " << bytes_per_second;
-
-    m_downloadSpeed = bytes_per_second;
-    setTransferChange(Tc_DownloadSpeed, true);
-
     //only start the verification search _after_ data has come in, that way only connections
     //are requested if there is already a successful one
-    if (bytes_per_second && !m_verificationSearch && MultiSegKioSettings::useSearchVerification()) {
+    if ((processedSize != m_downloadedSize) && !m_verificationSearch && MultiSegKioSettings::useSearchVerification()) {
         m_verificationSearch = true;
         QDomDocument doc;
         QDomElement element = doc.createElement("TransferDataSource");
@@ -242,6 +234,18 @@ void TransferMultiSegKio::slotSpeed(unsigned long bytes_per_second)
             checksumSearch->start();
         }
     }
+
+    m_downloadedSize = processedSize;
+
+    setTransferChange(Tc_DownloadedSize, true);
+}
+
+void TransferMultiSegKio::slotSpeed(unsigned long bytes_per_second)
+{
+    kDebug(5001) << "slotSpeed: " << bytes_per_second;
+
+    m_downloadSpeed = bytes_per_second;
+    setTransferChange(Tc_DownloadSpeed, true);
 }
 
 void TransferMultiSegKio::slotTotalSize(KIO::filesize_t size)
@@ -337,4 +341,9 @@ void TransferMultiSegKio::slotRename(const KUrl &oldUrl, const KUrl &newUrl)
     }
 }
 
+void TransferMultiSegKio::slotUpdateCapabilities()
+{
+    setCapabilities(m_dataSourceFactory->capabilities());
+}
+
 #include "transfermultisegkio.moc"
