@@ -25,7 +25,6 @@ class TransfersView : public QTreeView
         ~TransfersView();
 
         void setModel(QAbstractItemModel * model);
-        QModelIndex indexFromTransferHandler(TransferHandler *handler);
 
     private:
         void dropEvent(QDropEvent * event);
@@ -36,12 +35,23 @@ class TransfersView : public QTreeView
         void dragMoveEvent ( QDragMoveEvent * event );
         void rowsAboutToBeRemoved(const QModelIndex & parent, int start, int end);
 
+    public slots:
+        void closeExpandableDetails(const QModelIndex &index = QModelIndex());
+        void closeExpandableDetails(const QModelIndex &parent, int rowStart, int rowEnd);
+        void slotItemActivated(const QModelIndex & index);
+        void slotItemCollapsed(const QModelIndex & index);
+
     private slots:
         void toggleMainGroup();// show or hide the first group header if there's only one download group
-        void slotSetColumnVisible(int column);
         void slotShowHeaderMenu(const QPoint &point);
+        void selectionChanged(const QItemSelection & selected, const QItemSelection & deselected);
+        void slotHideSection(int logicalIndex);
+        void slotSectionMoved(int logicalIndex, int oldVisualIndex, int newVisualIndex);
 
     private:
+        QWidget *getDetailsWidgetForTransfer(TransferHandler *handler);
+
+        QList<QModelIndex> m_editingIndexes;
         KMenu *m_headerMenu;
 };
 
