@@ -82,12 +82,19 @@ ICQEditAccountWidget::ICQEditAccountWidget(ICQProtocol *protocol,
 
 		QString serverEntry = mAccount->configGroup()->readEntry("Server", "login.oscar.aol.com");
 		int portEntry = mAccount->configGroup()->readEntry("Port", 5190);
-		if ( serverEntry != "login.oscar.aol.com" || ( portEntry != 5190) )
-			mAccountSettings->optionOverrideServer->setChecked( true );
+		mAccountSettings->optionOverrideServer->setChecked( serverEntry != "login.oscar.aol.com" || ( portEntry != 5190) );
 
 		mAccountSettings->edtServerAddress->setText( serverEntry );
 		mAccountSettings->edtServerPort->setValue( portEntry );
 
+		bool proxyServerEnableEntry = mAccount->configGroup()->readEntry("ProxyEnable", false);
+		QString proxyServerEntry = mAccount->configGroup()->readEntry("ProxyServer", QString());
+		int proxyPortEntry = mAccount->configGroup()->readEntry("ProxyPort", 443);
+		mAccountSettings->optionEnableProxy->setChecked( proxyServerEnableEntry );
+		mAccountSettings->edtProxyServerAddress->setText( proxyServerEntry );
+		mAccountSettings->edtProxyServerPort->setValue( proxyPortEntry );
+
+
 		bool configChecked = mAccount->configGroup()->readEntry( "RequireAuth", false );
 		mAccountSettings->chkRequireAuth->setChecked( configChecked );
 
@@ -201,15 +208,9 @@ ICQEditAccountWidget::ICQEditAccountWidget(ICQProtocol *protocol,
 
 ICQEditAccountWidget::~ICQEditAccountWidget()
 {
-	if ( m_visibleEngine )
-		delete m_visibleEngine;
-	
-	if ( m_invisibleEngine )
-		delete m_invisibleEngine;
-	
-	if ( m_ignoreEngine )
-		delete m_ignoreEngine;
-	
+	delete m_visibleEngine;
+	delete m_invisibleEngine;
+	delete m_ignoreEngine;
 	delete mAccountSettings;
 }
 
@@ -222,8 +223,6 @@ Kopete::Account *ICQEditAccountWidget::apply()
 	{
 		kDebug(14153) << "Creating a new account";
 		mAccount = new ICQAccount(mProtocol, mAccountSettings->edtAccountId->text());
-		if(!mAccount)
-			return NULL;
 	}
 
 	mAccountSettings->mPasswordWidget->save(&mAccount->password());
@@ -258,6 +257,14 @@ Kopete::Account *ICQEditAccountWidget::apply()
 		mAccount->setServerPort(5190);
 	}
 
+	bool useProxy=mAccountSettings->optionEnableProxy->isChecked();
+	mAccount->setProxyServerEnabled( useProxy );
+	if ( mAccountSettings->optionEnableProxy->isChecked() )
+	{
+		mAccount->setProxyServerAddress(mAccountSettings->edtProxyServerAddress->text().trimmed());
+		mAccount->setProxyServerPort(mAccountSettings->edtProxyServerPort->value());
+	}
+
 	//set filetransfer stuff
 	configChecked = mAccountSettings->chkFileProxy->isChecked();
 	mAccount->configGroup()->writeEntry( "FileProxy", configChecked );
