@@ -1,7 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2007 - 2010 Urs Wolfer <uwolfer @ kde.org>
-** Copyright (C) 2009 Tony Murray <murraytony @ gmail.com>
+** Copyright (C) 2009 - 2010 Tony Murray <murraytony @ gmail.com>
 **
 ** This file is part of KDE.
 **
@@ -58,13 +58,13 @@
 #include <KStatusBar>
 #include <KToggleAction>
 #include <KToggleFullScreenAction>
-#include <KUrlNavigator>
 #include <KServiceTypeTrader>
 
 #include <QClipboard>
 #include <QDockWidget>
 #include <QFontMetrics>
 #include <QGroupBox>
+#include <QHBoxLayout>
 #include <QHeaderView>
 #include <QLabel>
 #include <QLayout>
@@ -73,11 +73,13 @@
 #include <QTableView>
 #include <QTimer>
 #include <QToolBar>
+#include <QVBoxLayout>
 
 MainWindow::MainWindow(QWidget *parent)
         : KXmlGuiWindow(parent),
         m_fullscreenWindow(0),
-        m_addressNavigator(0),
+        m_protocolInput(0),
+        m_addressInput(0),
         m_toolBar(0),
         m_currentRemoteView(-1),
         m_systemTrayIcon(0),
@@ -131,7 +133,7 @@ MainWindow::MainWindow(QWidget *parent)
     updateActionStatus(); // disable remote view actions
 
     if (Settings::openSessions().count() == 0) // just create a new connection tab if there are no open sessions
-        m_tabWidget->addTab(newConnectionWidget(), "New Connection");
+        m_tabWidget->addTab(newConnectionWidget(), i18n("New Connection"));
 
     setAutoSaveSettings(); // e.g toolbar position, mainwindow size, ...
 
@@ -284,11 +286,19 @@ void MainWindow::restoreOpenSessions()
     }
 }
 
+KUrl MainWindow::getInputUrl()
+{
+    KUrl inputUrl;
+    inputUrl.setAuthority(m_addressInput->text());
+    inputUrl.setProtocol(m_protocolInput->currentText());
+    return inputUrl;
+}
+
 void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConnected, const QString &tabName)
 {
     m_switchFullscreenWhenConnected = switchFullscreenWhenConnected;
 
-    const KUrl url = newUrl.isEmpty() ? m_addressNavigator->uncommittedUrl() : newUrl;
+    const KUrl url = newUrl.isEmpty() ? getInputUrl() : newUrl;
 
     if (!url.isValid() || (url.host().isEmpty() && url.port() < 0)
         || (!url.path().isEmpty() && url.path() != QLatin1String("/"))) {
@@ -298,8 +308,10 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
         return;
     }
 
-    if (m_addressNavigator) {
-        m_addressNavigator->setLocationUrl(url);
+    if (m_protocolInput && m_addressInput) {
+        int index = m_protocolInput->findText(url.protocol());
+        if (index>=0) m_protocolInput->setCurrentIndex(index);
+        m_addressInput->setText(url.authority());
     }
 
     RemoteView *view = 0;
@@ -461,7 +473,7 @@ void MainWindow::statusChanged(RemoteView::RemoteStatus status)
 
 void MainWindow::takeScreenshot()
 {
-    const QPixmap snapshot = QPixmap::grabWidget(m_remoteViewList.at(m_currentRemoteView));
+    const QPixmap snapshot = m_remoteViewList.at(m_currentRemoteView)->takeScreenshot();
 
     QApplication::clipboard()->setPixmap(snapshot);
 }
@@ -892,9 +904,6 @@ void MainWindow::preferences()
 
 void MainWindow::updateConfiguration()
 {
-    if (m_addressNavigator)
-        m_addressNavigator->setUrlEditable(Settings::normalUrlInputLine());
-
     if (!Settings::showStatusBar())
         statusBar()->deleteLater();
     else
@@ -1072,24 +1081,19 @@ QWidget* MainWindow::newConnectionWidget()
 
     {
         QHBoxLayout *connectLayout = new QHBoxLayout;
-        const QString initialProtocol(!m_remoteViewFactories.isEmpty() ? (*m_remoteViewFactories.begin())->scheme() : QString());
-        KUrl url;
-        url.setProtocol(initialProtocol);
-        url.setPath(QLatin1String("//"));
-        m_addressNavigator = new KUrlNavigator(0, url, m_newConnectionWidget);
-
-        QStringList schemes;
-        foreach(RemoteViewFactory *factory, m_remoteViewFactories) {
-            schemes << factory->scheme();
-        }
-        m_addressNavigator->setCustomProtocols(schemes);
-
-        m_addressNavigator->setUrlEditable(Settings::normalUrlInputLine());
-        connect(m_addressNavigator, SIGNAL(returnPressed()), SLOT(newConnection()));
-        m_addressNavigator->setFocus();
-        m_addressNavigator->setToolTip(i18n("Type an IP or DNS Name here. Clear the line to get a list of connection methods."));
 
         QLabel *addressLabel = new QLabel(i18n("Connect to:"), m_newConnectionWidget);
+        m_protocolInput = new KComboBox(m_newConnectionWidget);
+        m_addressInput = new KLineEdit(m_newConnectionWidget);
+        m_addressInput->setClearButtonShown(true);
+
+        foreach(RemoteViewFactory *factory, m_remoteViewFactories) {
+            m_protocolInput->addItem(factory->scheme());
+        }
+
+        connect(m_addressInput, SIGNAL(returnPressed()), SLOT(newConnection()));
+        m_addressInput->setFocus();
+        m_addressInput->setToolTip(i18n("Type an IP or DNS Name here. Clear the line to get a list of connection methods."));
 
         KPushButton *connectButton = new KPushButton(m_newConnectionWidget);
         connectButton->setToolTip(i18n("Goto Address"));
@@ -1097,7 +1101,8 @@ QWidget* MainWindow::newConnectionWidget()
         connect(connectButton, SIGNAL(clicked()), SLOT(newConnection()));
 
         connectLayout->addWidget(addressLabel);
-        connectLayout->addWidget(m_addressNavigator, 1);
+        connectLayout->addWidget(m_protocolInput);
+        connectLayout->addWidget(m_addressInput, 1);
         connectLayout->addWidget(connectButton);
         connectLayout->setContentsMargins(QMargins(0, 6, 0, 10));
         startLayout->addLayout(connectLayout);
@@ -1106,21 +1111,31 @@ QWidget* MainWindow::newConnectionWidget()
     {
         QGroupBox *remoteDesktopsGroupWidget = new QGroupBox(m_newConnectionWidget);
         QVBoxLayout *remoteDesktopsGroupLayout = new QVBoxLayout(remoteDesktopsGroupWidget);
-        QHBoxLayout *filterLayout = new QHBoxLayout;
+
+        // set up the filter input field
+        QHBoxLayout *filterLayout = new QHBoxLayout(remoteDesktopsGroupWidget);
+        QLabel *filterLabel = new QLabel(i18nc("Verb, to remove items that don't match", "Filter"), remoteDesktopsGroupWidget);
+        KLineEdit *filterLineEdit = new KLineEdit(remoteDesktopsGroupWidget);
+        filterLineEdit->setClickMessage(i18n("Type here to filter the connection list."));
+        filterLineEdit->setClearButtonShown(true);
+        connect(filterLineEdit, SIGNAL(textChanged(const QString &)), SLOT(updateFilter(const QString &)));
+        filterLayout->addWidget(filterLabel);
+        filterLayout->addWidget(filterLineEdit);
+        remoteDesktopsGroupLayout->addLayout(filterLayout);
+
         m_newConnectionTableView = new QTableView(remoteDesktopsGroupWidget);
         m_newConnectionTableView->setModel(m_remoteDesktopsModelProxy);
 
         // set up the view so it looks nice
         m_newConnectionTableView->setItemDelegate(new ConnectionDelegate(m_newConnectionTableView));
         m_newConnectionTableView->setShowGrid(false);
+        m_newConnectionTableView->setSelectionMode(QAbstractItemView::NoSelection);
         m_newConnectionTableView->verticalHeader()->hide();
         m_newConnectionTableView->verticalHeader()->setDefaultSectionSize(
             m_newConnectionTableView->fontMetrics().height() + 3);
         m_newConnectionTableView->horizontalHeader()->setStretchLastSection(true);
-//         m_newConnectionTableView->resizeColumnsToContents(); // not needed when resize mode is ResizeToContents
         m_newConnectionTableView->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
         m_newConnectionTableView->setAlternatingRowColors(true);
-
         // set up sorting and actions (double click open, right click custom menu)
         m_newConnectionTableView->setSortingEnabled(true);
         m_newConnectionTableView->sortByColumn(Settings::connectionListSortColumn(), Qt::SortOrder(Settings::connectionListSortOrder()));
@@ -1130,18 +1145,8 @@ QWidget* MainWindow::newConnectionWidget()
                 SLOT(openFromRemoteDesktopsModel(const QModelIndex &)));
         m_newConnectionTableView->setContextMenuPolicy(Qt::CustomContextMenu);
         connect(m_newConnectionTableView, SIGNAL(customContextMenuRequested(QPoint)), SLOT(showConnectionContextMenu(QPoint)));
-
-        // set up the filter input field
-        QLabel *filterLabel = new QLabel(i18nc("Verb, to remove items that don't match", "Filter"), remoteDesktopsGroupWidget);
-        KLineEdit *filterLineEdit = new KLineEdit(remoteDesktopsGroupWidget);
-        filterLineEdit->setClickMessage(i18n("Type here to filter the connection list."));
-        filterLineEdit->setClearButtonShown(true);
-        connect(filterLineEdit, SIGNAL(textChanged(const QString &)), SLOT(updateFilter(const QString &)));
-        filterLayout->addWidget(filterLabel);
-        filterLayout->addWidget(filterLineEdit);
-
-        remoteDesktopsGroupLayout->addLayout(filterLayout);
         remoteDesktopsGroupLayout->addWidget(m_newConnectionTableView);
+
         startLayout->addWidget(remoteDesktopsGroupWidget);
     }
 
@@ -1161,24 +1166,10 @@ void MainWindow::newConnectionPage()
     if (indexOfNewConnectionWidget >= 0)
         m_tabWidget->setCurrentIndex(indexOfNewConnectionWidget);
     else {
-        const int index = m_tabWidget->addTab(newConnectionWidget(), "New Connection");
+        const int index = m_tabWidget->addTab(newConnectionWidget(), i18n("New Connection"));
         m_tabWidget->setCurrentIndex(index);
     }
-#if 0 // not usable anymore, probably use it in another way? -uwolfer
-    QObject *senderObject = qobject_cast<QObject*>(sender());
-    if (senderObject) {
-        const QString scheme(senderObject->property("schemeString").toString());
-        if (!scheme.isEmpty()) {
-            m_addressNavigator->setUrl(KUrl(scheme + "://"));
-        }
-        const QString toolTip(senderObject->property("toolTipString").toString());
-        if (!toolTip.isEmpty()) {
-            QToolTip::showText(m_addressNavigator->pos() + pos() + QPoint(m_addressNavigator->width() / 2,
-                               m_addressNavigator->height() / 2), toolTip, this);
-        }
-    }
-#endif
-    m_addressNavigator->setFocus();
+    m_addressInput->setFocus();
 }
 
 QList<RemoteView *> MainWindow::remoteViewList() const
