@@ -26,12 +26,53 @@
 #include <QMutexLocker>
 #include <QTimer>
 
+//for detecting intel AMT KVM vnc server
+static const QString INTEL_AMT_KVM_STRING= "Intel(r) AMT KVM";
 static QString outputErrorMessageString;
 
+QVector<QRgb> VncClientThread::m_colorTable;
+
 rfbBool VncClientThread::newclient(rfbClient *cl)
 {
     VncClientThread *t = (VncClientThread*)rfbClientGetClientData(cl, 0);
     Q_ASSERT(t);
+    
+    //8bit color hack for Intel(r) AMT KVM "classic vnc" = vnc server built in in Intel Vpro chipsets.
+    if (INTEL_AMT_KVM_STRING == cl->desktopName) {
+        kDebug(5011) << "Intel(R) AMT KVM: switching to 8 bit color depth (workaround, recent libvncserver needed)";
+        t->setUse8BitColors(true);
+    }
+    if (t->use8BitColors()) {
+        if (m_colorTable.isEmpty()) {
+            m_colorTable.resize(256);
+            int r,g,b;
+            for (int i = 0; i < 256; ++i) {
+                //pick out the red (3 bits), green (3 bits) and blue (2 bits) bits and make them maximum significant in 8bits
+                //this gives a colortable for 8bit true colors
+                r= (i & 0x07) << 5;
+                g= (i & 0x38) << 2;
+                b= i & 0xc0;
+                m_colorTable[i] = qRgb(r, g, b);
+            }
+        }
+        cl->format.depth = 8;
+        cl->format.bitsPerPixel = 8;
+        cl->format.redShift = 0;
+        cl->format.greenShift = 3;
+        cl->format.blueShift = 6;
+        cl->format.redMax = 7;
+        cl->format.greenMax = 7;
+        cl->format.blueMax = 3;
+    } else {
+        cl->format.depth = 24;
+        cl->format.bitsPerPixel = 32;
+        cl->format.redShift = 16;
+        cl->format.greenShift = 8;
+        cl->format.blueShift = 0;
+        cl->format.redMax = 0xff;
+        cl->format.greenMax = 0xff;
+        cl->format.blueMax = 0xff;
+    }
 
     const int width = cl->width, height = cl->height, depth = cl->format.bitsPerPixel;
     const int size = width * height * (depth / 8);
@@ -40,23 +81,14 @@ rfbBool VncClientThread::newclient(rfbClient *cl)
     t->frameBuffer = new uint8_t[size];
     cl->frameBuffer = t->frameBuffer;
     memset(cl->frameBuffer, '\0', size);
-    cl->format.bitsPerPixel = 32;
-    cl->format.redShift = 16;
-    cl->format.greenShift = 8;
-    cl->format.blueShift = 0;
-    cl->format.redMax = 0xff;
-    cl->format.greenMax = 0xff;
-    cl->format.blueMax = 0xff;
 
     switch (t->quality()) {
     case RemoteView::High:
-        cl->appData.useBGR233 = 0;
         cl->appData.encodingsString = "copyrect hextile raw";
         cl->appData.compressLevel = 0;
         cl->appData.qualityLevel = 9;
         break;
     case RemoteView::Medium:
-        cl->appData.useBGR233 = 0;
         cl->appData.encodingsString = "tight zrle ultra copyrect hextile zlib corre rre raw";
         cl->appData.compressLevel = 5;
         cl->appData.qualityLevel = 7;
@@ -64,32 +96,34 @@ rfbBool VncClientThread::newclient(rfbClient *cl)
     case RemoteView::Low:
     case RemoteView::Unknown:
     default:
-        cl->appData.useBGR233 = 1;
         cl->appData.encodingsString = "tight zrle ultra copyrect hextile zlib corre rre raw";
         cl->appData.compressLevel = 9;
         cl->appData.qualityLevel = 1;
     }
 
-    SetFormatAndEncodings(cl);
-
+    kDebug(5011) << "Client created";
     return true;
 }
 
 void VncClientThread::updatefb(rfbClient* cl, int x, int y, int w, int h)
 {
 //     kDebug(5011) << "updated client: x: " << x << ", y: " << y << ", w: " << w << ", h: " << h;
+    VncClientThread *t = (VncClientThread*)rfbClientGetClientData(cl, 0);
+    Q_ASSERT(t);
 
     const int width = cl->width, height = cl->height;
-
-    const QImage img(cl->frameBuffer, width, height, QImage::Format_RGB32);
+    QImage img;
+    if (t->use8BitColors()) {
+        img = QImage(cl->frameBuffer, width, height, QImage::Format_Indexed8);
+        img.setColorTable(m_colorTable);
+    } else {
+        img = QImage(cl->frameBuffer, width, height, QImage::Format_RGB32);
+    }
 
     if (img.isNull()) {
         kDebug(5011) << "image not loaded";
     }
 
-    VncClientThread *t = (VncClientThread*)rfbClientGetClientData(cl, 0);
-    Q_ASSERT(t);
-
     t->setImage(img);
 
     t->emitUpdated(x, y, w, h);
@@ -157,9 +191,10 @@ void VncClientThread::outputHandler(const char *format, ...)
 VncClientThread::VncClientThread(QObject *parent)
         : QThread(parent)
         , frameBuffer(0)
+        , m_stopped(false)
+        , cl(0)
 {
     QMutexLocker locker(&mutex);
-    m_stopped = false;
 
     QTimer *outputErrorMessagesCheckTimer = new QTimer(this);
     outputErrorMessagesCheckTimer->setInterval(500);
@@ -169,12 +204,18 @@ VncClientThread::VncClientThread(QObject *parent)
 
 VncClientThread::~VncClientThread()
 {
-    stop();
+    if(isRunning()) {
+        stop();
+        terminate();
+        const bool quitSuccess = wait(1000);
+        kDebug(5011) << "Attempting to stop in deconstructor, will crash if this fails:" << quitSuccess;
+    }
 
-    const bool quitSuccess = wait(500);
+    if (cl) {
+        // Disconnect from vnc server & cleanup allocated resources
+        rfbClientCleanup(cl);
+    }
 
-    kDebug(5011) << "Quit VNC thread success:" << quitSuccess;
-    
     delete [] frameBuffer;
 }
 
@@ -207,11 +248,21 @@ void VncClientThread::setQuality(RemoteView::Quality quality)
     m_quality = quality;
 }
 
+void VncClientThread::setUse8BitColors(bool use8BitColors)
+{
+    m_use8BitColors= use8BitColors;
+}
+
 RemoteView::Quality VncClientThread::quality() const
 {
     return m_quality;
 }
 
+bool VncClientThread::use8BitColors() const
+{
+    return m_use8BitColors;
+}
+
 void VncClientThread::setImage(const QImage &img)
 {
     QMutexLocker locker(&mutex);
@@ -249,10 +300,13 @@ void VncClientThread::run()
     QMutexLocker locker(&mutex);
 
     while (!m_stopped) { // try to connect as long as the server allows
+        locker.relock();
         m_passwordError = false;
+        locker.unlock();
 
         rfbClientLog = outputHandler;
         rfbClientErr = outputHandler;
+        //24bit color dept in 32 bits per pixel = default. Will change colordepth and bpp later if needed
         cl = rfbGetClient(8, 3, 4);
         cl->MallocFrameBuffer = newclient;
         cl->canHandleNewFBSize = true;
@@ -261,6 +315,7 @@ void VncClientThread::run()
         cl->GotXCutText = cuttext;
         rfbClientSetClientData(cl, 0, this);
 
+        locker.relock();
         cl->serverHost = strdup(m_host.toUtf8().constData());
 
         if (m_port < 0 || !m_port) // port is invalid or empty...
@@ -269,43 +324,44 @@ void VncClientThread::run()
         if (m_port >= 0 && m_port < 100) // the user most likely used the short form (e.g. :1)
             m_port += 5900;
         cl->serverPort = m_port;
+        locker.unlock();
 
         kDebug(5011) << "--------------------- trying init ---------------------";
 
         if (rfbInitClient(cl, 0, 0))
             break;
+        else
+            cl = 0;
 
+        locker.relock();
         if (m_passwordError)
             continue;
 
         return;
     }
 
-    locker.unlock();
-
-    // Main VNC event loop
+    locker.relock();
+    kDebug(5011) << "--------------------- Starting main VNC event loop ---------------------";
     while (!m_stopped) {
+        locker.unlock();
         const int i = WaitForMessage(cl, 500);
-        if (i < 0)
+        if (i < 0) {
             break;
-        if (i)
-            if (!HandleRFBServerMessage(cl))
+        }
+        if (i) {
+            if (!HandleRFBServerMessage(cl)) {
                 break;
-
-        locker.relock();
+            }
+        }
 
         while (!m_eventQueue.isEmpty()) {
             ClientEvent* clientEvent = m_eventQueue.dequeue();
             clientEvent->fire(cl);
             delete clientEvent;
         }
-
-        locker.unlock();
+        locker.relock();
     }
 
-    // Cleanup allocated resources
-    locker.relock();
-    rfbClientCleanup(cl);
     m_stopped = true;
 }
 
