@@ -85,14 +85,6 @@ VncView::VncView(QWidget *parent, const KUrl &url, KConfigGroup configGroup)
 
 VncView::~VncView()
 {
-    unpressModifiers();
-
-    // Disconnect all signals so that we don't get any more callbacks from the client thread
-    disconnect(&vncThread, SIGNAL(imageUpdated(int, int, int, int)), this, SLOT(updateImage(int, int, int, int)));
-    disconnect(&vncThread, SIGNAL(gotCut(const QString&)), this, SLOT(setCut(const QString&)));
-    disconnect(&vncThread, SIGNAL(passwordRequest()), this, SLOT(requestPassword()));
-    disconnect(&vncThread, SIGNAL(outputErrorMessage(QString)), this, SLOT(outputErrorMessage(QString)));
-
     startQuitting();
 }
 
@@ -163,10 +155,20 @@ void VncView::startQuitting()
 {
     kDebug(5011) << "about to quit";
 
-    const bool connected = status() == RemoteView::Connected;
-
     setStatus(Disconnecting);
 
+    unpressModifiers();
+
+    // Disconnect all signals so that we don't get any more callbacks from the client thread
+    bool imageUpdatedDisconnect = disconnect(&vncThread, SIGNAL(imageUpdated(int, int, int, int)), this, SLOT(updateImage(int, int, int, int)));
+    bool gotCutDisconnect = disconnect(&vncThread, SIGNAL(gotCut(const QString&)), this, SLOT(setCut(const QString&)));
+    bool passwordRequestDisconnect = disconnect(&vncThread, SIGNAL(passwordRequest()), this, SLOT(requestPassword()));
+    bool outputErrorMessageDisconnect = disconnect(&vncThread, SIGNAL(outputErrorMessage(QString)), this, SLOT(outputErrorMessage(QString)));
+
+    kDebug(5011) << "Signals disconnected: imageUpdated: " << imageUpdatedDisconnect << "gotCut: " << gotCutDisconnect << "passwordRequest: " << passwordRequestDisconnect << "outputErrorMessage: " << outputErrorMessageDisconnect;
+
+    const bool connected = status() == RemoteView::Connected;
+
     m_quitFlag = true;
 
     if (connected) {
@@ -175,7 +177,7 @@ void VncView::startQuitting()
 
     vncThread.quit();
 
-    const bool quitSuccess = vncThread.wait(500);
+    const bool quitSuccess = vncThread.wait(1000);
 
     kDebug(5011) << "Quit VNC thread success:" << quitSuccess;
 
@@ -192,14 +194,20 @@ bool VncView::start()
     vncThread.setHost(m_host);
     vncThread.setPort(m_port);
     RemoteView::Quality quality;
+    bool use8BitColors;
 #ifdef QTONLY
     quality = (RemoteView::Quality)((QCoreApplication::arguments().count() > 2) ?
         QCoreApplication::arguments().at(2).toInt() : 2);
+    use8BitColors = ((QCoreApplication::arguments().count() > 3) ?
+        (QCoreApplication::arguments().at(3) == "8bit") : false);
 #else
     quality = m_hostPreferences->quality();
+    //don't use 8bit colors by default
+    use8BitColors = false;
 #endif
 
     vncThread.setQuality(quality);
+    vncThread.setUse8BitColors(use8BitColors);
 
     // set local cursor on by default because low quality mostly means slow internet connection
     if (quality == RemoteView::Low) {
@@ -336,14 +344,19 @@ void VncView::updateImage(int x, int y, int w, int h)
         setMouseTracking(true); // get mouse events even when there is no mousebutton pressed
         setFocusPolicy(Qt::WheelFocus);
         setStatus(Connected);
-//         emit framebufferSizeChanged(m_frame.width(), m_frame.height());
         emit connected();
         
         if (m_scale) {
-            if (parentWidget())
-                scaleResize(parentWidget()->width(), parentWidget()->height());
-        } 
-        
+#ifndef QTONLY
+            kDebug(5011) << "Setting initial size w:" <<m_hostPreferences->width() << " h:" << m_hostPreferences->height();
+            emit framebufferSizeChanged(m_hostPreferences->width(), m_hostPreferences->height());
+            scaleResize(m_hostPreferences->width(), m_hostPreferences->height());
+            kDebug() << "m_frame.size():" << m_frame.size() << "size()" << size();
+#else
+//TODO: qtonly alternative
+#endif
+        }
+
         m_initDone = true;
 
 #ifndef QTONLY
@@ -364,8 +377,8 @@ void VncView::updateImage(int x, int y, int w, int h)
             resize(m_frame.width(), m_frame.height());
             setMaximumSize(m_frame.width(), m_frame.height()); //This is a hack to force Qt to center the view in the scroll area
             setMinimumSize(m_frame.width(), m_frame.height());
+            emit framebufferSizeChanged(m_frame.width(), m_frame.height());
         }
-        emit framebufferSizeChanged(m_frame.width(), m_frame.height());
     }
 
     m_repaint = true;
